<?php

/**
 *
 * Category model for the cart
 *
 * @package	VirtueMart
 * @subpackage Cart
 * @author RolandD
 * @author Max Milbers
 * @link ${PHING.VM.MAINTAINERURL}
 * @copyright Copyright (c) 2004 - 2010 VirtueMart Team. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * @version $Id: cart.php 9464 2017-03-06 11:00:33Z Milbo $
 */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');


/**
 * Model class for the cart
 * Very important, use ALWAYS the getCart function, to get the cart from the session
 * @package	VirtueMart
 * @subpackage Cart
 * @author RolandD
 * @author Max Milbers
 */
class VirtueMartCart {

	var $products = array();
	var $_productAdded = true;
	var $_calculated = false;
	var $_inCheckOut = false;
	var $_inConfirm = false;
	var $_fromCart = false;
	var $_dataValidated = false;
	var $_blockConfirm = false;
	var $_confirmDone = false;
	var $_redirect = false;
	var $_redirected = false;
	var $_redirect_disabled = false;
	var $_lastError = null; // Used to pass errmsg to the cart using addJS()
	//todo multivendor stuff must be set in the add function, first product determines ownership of cart, or a fixed vendor is used
	var $vendorId = 0;
	var $lastVisitedCategoryId = 0;
	var $virtuemart_shipmentmethod_id = 0;
	var $virtuemart_paymentmethod_id = 0;
	var $automaticSelectedShipment = false;
	var $automaticSelectedPayment  = false;
	var $BT = 0;
	var $ST = 0;
	var $cartfields = null;

	var $couponCode = '';
	var $order_language = '';
	var $orderDetails = 0;

	var $lists = null;
	var $order_number=null; // added to solve emptying cart for payment notification
	var $order_pass=null;
	var $virtuemart_order_id = false;
	var $customer_number=null;
	var $cartAdv = false;

	var $pricesCurrency = null;
	var $paymentCurrency = null;
	var $STsameAsBT = 1;
	var $selected_shipto = 0;
	var $productParentOrderable = TRUE;
	var $_triesValidateCoupon = array();

	var $cartProductsData = array();
	var $cartData = array();
	var $cartPrices = array();
	var $layout ;
	var $layoutPath='';
	var $virtuemart_cart_id = 0;
	var $customer_notified = false;
	/* @deprecated */
	var $pricesUnformatted = array();

	private static $_cart = null;

	var $useSSL = 1;

	public function __construct() {
		$this->useSSL = vmURI::useSSL();
		$this->useXHTML = false;
		$this->cartProductsData = array();
		$this->layout = VmConfig::get('cartlayout','default');

		if(empty($this->layout)){
			$this->layout = 'default';
		}
	}
	
	
	//Nanda Begin 
	function calc($str) {     
		$float = 0;     
		$float_str = $str;         
		if(isset($float_str) && $float_str!="" && $float_str!=null){
			list($top, $bottom) = explode('/', $float_str);    
			if($bottom!=0)		
			$float = $top / $bottom;  
		}
		return $float; 
	}


	//Nanda - Need to return price without discounts if the selection is cut or join as QBs cannot be applied on them
	function getQBPrice($product_id,$QBValue, $price_type, $chop_price, $join_price, $sheet_price, $qb1_price, $qb2_price, $qb3_price, $product_price)
	{
		$item=0;

		vmdebug('product_id:',$product_id);
		vmdebug('QBValue:',$QBValue);
		vmdebug('price_type:',$price_type);
		vmdebug('chop_price:',$chop_price);
		vmdebug('join_price:',$join_price);
		vmdebug('sheet_price:',$sheet_price);
		vmdebug('qb1_price:',$qb1_price);
		vmdebug('qb2_price:',$qb2_price);
		vmdebug('qb3_price:',$qb3_price);
		vmdebug('product_price:',$product_price);
		
		
		$user = JFactory::getUser();
		if($user->id > 0){
		
			$group_id = 3;
			$group_name = "Artist";
			$userGroups = $user->getAuthorisedGroups();

			for($i=0; $i<count($userGroups); $i++){
				if($userGroups[$i]==$group_id) break;
			}
			if($userGroups[$i]==$group_id) {
				$group_name = "Wholesale";
			} else {
				$group_name = "Artist";
			}
		
			//Hard coding artist user percentage as 10% for now, TODO - change it to add as configuration and get from config data
			//$artist=10.0;
			$artist=VmConfig::get('artist_user_config',0);
		
			//If the unit of measure is either chop (2) or join (3), then just return the 
			//base price of chop or join without any discounts
			if($price_type=='2'){
				$item=$chop_price;
				
				if($group_name == "Artist"){
					$item= $item + ($item*$artist/100);
				}
				return $item;
			}
			elseif($price_type=='3'){
				$item=$join_price;

				if($group_name == "Artist"){
					$item= $item + ($item*$artist/100);
				}
				
				return $item;
			}
			elseif($price_type=='4'){
				$item=$sheet_price;
				
				if($group_name == "Artist"){
					$item= $item + ($item*$artist/100);
				}
				
				return $item;
			}	

			if($QBValue==1 && $qb1_price!=0) {
				$item = $qb1_price;
			}
			elseif($QBValue==2 && $qb2_price!=0) {
				$item = $qb2_price;
			}
			elseif($QBValue==3 && $qb3_price!=0){
				$item = $qb3_price;
			}
			else{
				$item=$product_price;
				
				if($group_name == "Artist"){
					$item= $item + ($item*$artist/100);
				}
			}
		}
		return $item;
	}


	//Nanda 01-25-2012
	//Function to get actual quantity of specified product in cart
	//Actual quantity refers to calculated footage x number of items
	function getActualQuantity($tmpProduct){
		$pid = $tmpProduct->virtuemart_product_id;
		if(isset($tmpProduct->purchase_type))
			$price_type=$tmpProduct->purchase_type;
		/*else
			$price_type='1';*/
		
		//Calculation for Wood and Metal
		if($tmpProduct->virtuemart_category_id=='1' || $tmpProduct->virtuemart_category_id=='7'){	
			if(isset($tmpProduct->base1))
				$base1=(integer)$tmpProduct->base1;
			else
				$base1=1;

			if(isset($tmpProduct->fraction1)){
				$fraction1= self::$_cart->calc($tmpProduct->fraction1);
			}
			else{
				$fraction1=0;
			}
			
			$base1 = $base1+$fraction1;
			if(isset($tmpProduct->base2)){
				$base2=(integer)$tmpProduct->base2;
			}
			else{
				$base2=0;
			}
			
			if(isset($tmpProduct->fraction2)){
				$fraction2=self::$_cart->calc($tmpProduct->fraction2);
			}
			else{
				$fraction2=0;
			}
			
			$base2=$base2+$fraction2;
			 
			if($price_type!=1){
				$base=$base1+$base2;
				$width=$tmpProduct->product_width;
				$width=8*$width;

				if($width<4){
					$dimcal=(($base*2)+(4))/12;
				}
				else{
					$dimcal=(($base*2)+($width))/12;
				}
				//Condition adding to convert dimcal to 4 if dimcal is less than 4
              
				if($dimcal>11){
					$dimcal=ceil($dimcal);
				}
				else{
					if((ceil($dimcal) - $dimcal) >0.5 )
						$dimcal=ceil($dimcal)-0.5;
					else
						$dimcal=ceil($dimcal);
				}
				if($dimcal<4){
					$dimcal=4;
				}
			}
			else{
				$dimcal=$base1;
			}
		}
		else if($tmpProduct->virtuemart_category_id=='3'||$tmpProduct->virtuemart_category_id=='4'||$tmpProduct->virtuemart_category_id=='5') //Calculations for Glass, Plexi and Mirror
		{
			if(isset($tmpProduct->base1))
				$base1=(integer)$tmpProduct->base1;
			else
				$base1=1;
				
			if(isset($tmpProduct->fraction1)){
				$fraction1=self::$_cart->calc($tmpProduct->fraction1);
			}
			else{
				$fraction1=0;
			}
			
			$base1= $base1+$fraction1;

			if(isset($tmpProduct->base2)){
				$base2=(integer)$tmpProduct->base2;
			}
			else{
				$base2=0;
			}
			
			if(isset($tmpProduct->fraction2)){
				$fraction2=self::$_cart->calc($tmpProduct->fraction2);
			}
			else{
				$fraction2=0;
			}

			$base2=$base2+$fraction2;
			$base=round($base1,2)*round($base2,2);
			
			$dimcal=ceil($base/144);
		}
		else{
			$dimcal=1;
		} 
		
		if($dimcal==0)
			$dimcal=1;		
			
		return ($tmpProduct->quantity * $dimcal);
	}

function getQB($PGQuantity,$GroupId) {
	
	vmdebug('PGQuantity:',$PGQuantity);
	vmdebug('GroupId:',$GroupId);

		$user = JFactory::getUser();
		$is_PC1 = false;
		if($user->id > 0){
		
			$group_id = 3;
			$group_name = "Artist";
			$userGroups = $user->getAuthorisedGroups();
			//vmdebug('$userGroups',$userGroups);

			for($i=0; $i<count($userGroups); $i++){
				if($userGroups[$i]==$group_id) break;
			}
			if($userGroups[$i]==$group_id) {
				$group_name = "Wholesale";
			} else {
				$group_name = "Artist";
			}


			//Check for PC1 user
			$is_PC1 = false;
			foreach ($userGroups as $value) {
				if($value==4) $is_PC1 = true;
			}
		}
	$QB=0;
	$qblist=Array();
	$pgM =VmModel::getModel('pricegroup');
	$pg = $pgM->getQB1($GroupId);
	if(count($pg)>0){
		$qb1=$pg[0];
	}
 	$pgM =VmModel::getModel('pricegroup');
	$pg = $pgM->getQB2($GroupId);
	if(count($pg)>0){
		$qb2=$pg[0];
	}
	$pgM =VmModel::getModel('pricegroup');
	$pg = $pgM->getQB3($GroupId);
	if(count($pg)>0){
		$qb3=$pg[0];
	}	
	vmdebug('qb1:',$qb1);
	vmdebug('qb2:',$qb2);
	vmdebug('qb3:',$qb3);
	if(isset($qb1)) {
		$qbM =VmModel::getModel('qb');
		$qbD1 = $qbM->getQBDetails($qb1);
		if(isset($qbD1)){
			$qblist[0]=$qbD1->quantity;
			$qblist[1]=$qbD1->startdate;
			$qblist[2]=$qbD1->enddate;
		}	
	}

	if(isset($qb2)) {
		$qbM =VmModel::getModel('qb');
		$qbD2 = $qbM->getQBDetails($qb2);
		if(isset($qbD2)){
			$qblist[3]=$qbD2->quantity;
			$qblist[4]=$qbD2->startdate;
			$qblist[5]=$qbD2->enddate;
		}
	}
	
	if(isset($qb3)) {
		$qbM =VmModel::getModel('qb');
		$qbD3 = $qbM->getQBDetails($qb3);
		if(isset($qbD3)){
			$qblist[6]=$qbD3->quantity;
			$qblist[7]=$qbD3->startdate;
			$qblist[8]=$qbD3->enddate;
		}
	}

	$todays_date = date("Y-m-d");
vmdebug('qblist:',$qblist);
	if( ($PGQuantity>=$qblist[0] || $is_PC1)&&$todays_date>=$qblist[1]&&$todays_date<=$qblist[2])
		$QB=1;
	if(($PGQuantity>=$qblist[3] || $is_PC1)&&$todays_date>=$qblist[4]&&$todays_date<=$qblist[5])
		$QB=2;
	if(($PGQuantity>=$qblist[6] || $is_PC1)&&$todays_date>=$qblist[7]&&$todays_date<=$qblist[8])
		$QB=3;
vmdebug('QB:',$QB);
	return $QB;
}	
	
	/**
	 * Get the cart from the session
	 *
	 * @author Max Milbers
	 * @access public
	 * @param array $cart the cart to store in the session
	 */
	public static function &getCart($forceNew=false, $options = array(), $cartData=NULL) {
vmdebug('getCart function call:');
		//What does this here? for json stuff?
		if (!class_exists('JTable')) require(VMPATH_LIBS . DS . 'joomla' . DS . 'database' . DS . 'table.php');
		//vmdebug("NN1",$forceNew);
		if(empty(self::$_cart) or $forceNew){

			self::$_cart = new VirtueMartCart;
			//vmdebug("NN2",$cartData);
			if (empty($cartData)) {
				$session = JFactory::getSession($options);
				//vmdebug("NN222222.1",$options);
				$cartSession = $session->get('vmcart', 0, 'vm');
				if (empty($cartSession)) {
					$user = JFactory::getUser();
					// get Joomla database object
					$db = JFactory::getDBO();
					$query = $db->getQuery(true);
					$query->select('cartData')
						->from($db->quoteName('#__virtuemart_carts'))
						->where('virtuemart_user_id='.$user->id);
					$db->setQuery($query);
					try{
						$cartSession = $db->loadResult();
					}
					catch (Exception $e){
						 // To show any exception if occured
						 //return JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
					}
				}
				//vmdebug("NN222222.2",$cartSession);
				if (!empty($cartSession)) {
					$sessionCart = (object)json_decode( $cartSession ,true);
					//vmdebug("NN222222.3",$sessionCart);
					if(empty($sessionCart->cartProductsData) or ($sessionCart->_guest and $sessionCart->_guest!=JFactory::getUser()->guest)){
					//vmdebug("NN222222.4",$sessionCart->cartProductsData);
						self::$_cart->loadCart($sessionCart);
					}
				}
			} else {
				$cartSession=$sessionCart=$cartData;
			}

			$userModel = VmModel::getModel('user');
			self::$_cart->user = $userModel->getCurrentUser();

			$lang = JFactory::getLanguage();
			self::$_cart->order_language = $lang->getTag();

			//NANDA
			//vmdebug("NN3:",$cartSession);
			
			if (!empty($cartSession)) {

				if(isset($sessionCart->cartProductsData)){
					self::$_cart->cartProductsData 				= $sessionCart->cartProductsData;
					self::$_cart->vendorId	 					= $sessionCart->vendorId;
					self::$_cart->lastVisitedCategoryId	 		= $sessionCart->lastVisitedCategoryId;
					self::$_cart->virtuemart_shipmentmethod_id	= $sessionCart->virtuemart_shipmentmethod_id;
					self::$_cart->virtuemart_paymentmethod_id 	= $sessionCart->virtuemart_paymentmethod_id;
					self::$_cart->automaticSelectedShipment 	= $sessionCart->automaticSelectedShipment;
					self::$_cart->automaticSelectedPayment 		= $sessionCart->automaticSelectedPayment;
					self::$_cart->BT 							= $sessionCart->BT;
					self::$_cart->ST 							= $sessionCart->ST;
					self::$_cart->cartfields					= $sessionCart->cartfields;

					self::$_cart->couponCode 					= $sessionCart->couponCode;
					self::$_cart->_triesValidateCoupon			= $sessionCart->_triesValidateCoupon;
					self::$_cart->order_number					= $sessionCart->order_number;
					self::$_cart->pricesCurrency				= $sessionCart->pricesCurrency;
					self::$_cart->paymentCurrency				= $sessionCart->paymentCurrency;

					self::$_cart->_guest						=  $sessionCart->_guest;
					self::$_cart->_inCheckOut 					= $sessionCart->_inCheckOut;
					self::$_cart->_inConfirm					= $sessionCart->_inConfirm;
					self::$_cart->_dataValidated				= $sessionCart->_dataValidated;
					self::$_cart->_confirmDone					= $sessionCart->_confirmDone;
					self::$_cart->STsameAsBT					= $sessionCart->STsameAsBT;
					self::$_cart->selected_shipto 				= $sessionCart->selected_shipto;
					self::$_cart->_fromCart						= $sessionCart->_fromCart;
					self::$_cart->layout						= $sessionCart->layout;
					self::$_cart->layoutPath				    = $sessionCart->layoutPath;
					self::$_cart->virtuemart_cart_id			= $sessionCart->virtuemart_cart_id;
				}
			}

			self::$_cart->loadSetRenderBTSTAddress();

			if (empty(self::$_cart->virtuemart_shipmentmethod_id) && !empty(self::$_cart->user->virtuemart_shipmentmethod_id)) {
				self::$_cart->virtuemart_shipmentmethod_id = self::$_cart->user->virtuemart_shipmentmethod_id;
			}

			if (empty(self::$_cart->virtuemart_paymentmethod_id) && !empty(self::$_cart->user->virtuemart_paymentmethod_id)) {
				self::$_cart->virtuemart_paymentmethod_id = self::$_cart->user->virtuemart_paymentmethod_id;
			}

			if((!empty(self::$_cart->user->tos) || !empty(self::$_cart->BT['tos'])) && !VmConfig::get('agree_to_tos_onorder',0) ){
				self::$_cart->BT['tos'] = 1;
			}
			
			//NANDA
			if(empty(self::$_cart->BT['email'])  ){
				$user = JFactory::getUser();
				self::$_cart->BT['email'] = $user->email;
			}

			if(!empty(self::$_cart->user->customer_number)){
				self::$_cart->customer_number = self::$_cart->user->customer_number;
			}

			if(empty(self::$_cart->customer_number) or strpos(self::$_cart->customer_number,'nonreg_')!==FALSE){
				$firstName = empty(self::$_cart->BT['first_name'])? '':self::$_cart->BT['first_name'];
				$lastName = empty(self::$_cart->BT['last_name'])? '':self::$_cart->BT['last_name'];
				$email = empty(self::$_cart->BT['email'])? '':self::$_cart->BT['email'];
				self::$_cart->customer_number = 'nonreg_'.$firstName.$lastName.$email;
			}

			$multixcart = VmConfig::get('multixcart',0);
			if(!empty($multixcart)){
				if($multixcart=='byvendor' or self::$_cart->vendorId==1){
					$vendor = VmModel::getModel('vendor');
					$vId = $vendor->getLoggedVendor();
					if(!empty($vId) and $vId!=1){
						self::$_cart->vendorId = $vId;
					}
					if(empty(self::$_cart->vendorId)) self::$_cart->vendorId = 1;
				}
				if($multixcart=='byselection'){
					self::$_cart->vendorId = vRequest::getInt('virtuemart_vendor_id',self::$_cart->vendorId);
				}
			}

			//We need to check for the amount of products. A cart in Multix mode using the first product
			// to determine the vendorId is a valid if there is no product in the cart
			$cp = count(self::$_cart->cartProductsData);
			if( $cp >0 and empty(self::$_cart->vendorId)){
				self::$_cart->vendorId = 1;
			} else if ($cp == 0 and !empty($multixcart)) {
				self::$_cart->vendorId = 0;
			}
			//vmdebug('Created new cart',self::$_cart->vendorId);
		}
		
		//NANDA
		//vmdebug('Cart:',self::$_cart->cartProductsData[0][base1]);
		//vmdebug('Cart123:',self::$_cart->products[0]->virtuemart_product_id);
		
		//vmdebug('User', $user->id);
		$user = JFactory::getUser();
		if($user->id > 0){
		
			$group_id = 3;
			$group_name = "Artist";
			$userGroups = $user->getAuthorisedGroups();
			//vmdebug('$userGroups',$userGroups);

			for($i=0; $i<count($userGroups); $i++){
				if($userGroups[$i]==$group_id) break;
			}
			if($userGroups[$i]==$group_id) {
				$group_name = "Wholesale";
			} else {
				$group_name = "Artist";
			}


			//Check for PC1 user
			$is_PC1 = false;
			foreach ($userGroups as $value) {
				if($value==4) $is_PC1 = true;
			}

			//vmdebug('$is_PC1', 'TT:'.$is_PC1);
			
			//Hard coding artist user percentage as 10% for now, TODO - change it to add as configuration and get from config data
			//$artist=10.0;
			$artist=VmConfig::get('artist_user_config',0);
		
			foreach (self::$_cart->cartProductsData as $cartProductData) {
				foreach (self::$_cart->products as $tmpProduct) {
					if(($cartProductData[virtuemart_product_id]==$tmpProduct->virtuemart_product_id) && ($cartProductData[purchase_types]==$tmpProduct->purchase_type)) {
					
						if (isset($cartProductData[purchase_type])) {
							$tmpProduct->purchase_type = $cartProductData[purchase_type];
						}				
						if (isset($cartProductData[base1])) {
							$tmpProduct->base1 = $cartProductData[base1];
						}
						if (isset($cartProductData[fraction1])) {
							$tmpProduct->fraction1 = $cartProductData[fraction1];
						}
						if (isset($cartProductData[base2])) {
							$tmpProduct->base2 = $cartProductData[base2];
						}
						if (isset($cartProductData[fraction2])) {
							$tmpProduct->fraction2 = $cartProductData[fraction2];
						}
						
						break;
					}
				}

			}
			
			//vmdebug("PRPR:",self::$_cart->products);
	

			//Nanda Add
			$PGList=Array();
			$PGQBList=Array();
			$PGItemQBPrice=Array();
			$ItemPriceFlag=Array();
			
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				
				$productM =VmModel::getModel('product');
				$pc = $productM->getPC($product_id);
				
				if(count($pc)>0){
					$pcM =VmModel::getModel('priceclass');
					$pg = $pcM->GetPG($pc[0]);
					if(count($pg)>0){
						$PGid = $pg[0];
						//vmdebug("tmpProduct:",$tmpProduct);
						
						if(isset ($PGList[$PGid])){
							$PGList[$PGid]=$PGList[$PGid] + self::$_cart->getActualQuantity($tmpProduct);
						}
						else{
							$PGList[$PGid]=	self::$_cart->getActualQuantity($tmpProduct);
						}
					}
				}
			}

				
			reset($PGList);
			while (list($key, $val) = each($PGList)) {
				$PGQBList[$key]=self::$_cart->getQB($PGList[$key],$key);
			}		
				vmdebug("PGList:",$PGList);
				vmdebug("PGQBList:",$PGQBList);
				
			$i=0;
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				//$pid = '__'.$i.'__'.$product_id;
				$pid = '__'.$i.'__'.$product_id.'__'.$tmpProduct->purchase_type;
				
				$price_type=null;

				if(isset($tmpProduct->purchase_type))
					$price_type=$tmpProduct->purchase_type;
				/*else
					$price_type='1';		*/

				$productM =VmModel::getModel('product');
				$pc = $productM->getPC($product_id);
				
				unset($PGid);
				if(count($pc)>0){
					$pcM =VmModel::getModel('priceclass');
					$pg = $pcM->GetPG($pc[0]);
					if(count($pg)>0){
						$PGid = $pg[0];
					}
				}
			 
				//If chop or join is selected, need to ignore QB price, hence forcibly set the flag as no-discount
				if($price_type=='2' || $price_type=='3'){
					$PGQBList[$PGid]=0;
				}
				
				if(count($tmpProduct->customfields)<=0) {
					$customFieldsModel = VmModel::getModel('customfields');
					$tmpProduct->customfields = $customFieldsModel->getCustomEmbeddedProductCustomFields($tmpProduct->allIds,0,-1);
				}
				
				//Get custom fields data and loop through to note the chop, join and QB pricing details
				foreach($tmpProduct->customfields as $customfield){
					if($customfield->virtuemart_custom_id==17) {
						$chop_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==18) {
						$join_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==19) {
						$sheet_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==20) {
						$qb1_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==21) {
						$qb2_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==22) {
						$qb3_price = $customfield->customfield_value; 
					}
				}
				
				$PGItemQBPrice[$pid]=self::$_cart->getQBPrice($product_id,$PGQBList[$PGid], $price_type, $chop_price, $join_price, $sheet_price, $qb1_price, $qb2_price, $qb3_price, $tmpProduct->prices['product_price']);
				
				if($PGQBList[$PGid]==1){
					$ItemPriceFlag[$pid] = 1;
				}
				elseif($PGQBList[$PGid]==2){
					$ItemPriceFlag[$pid] = 2;
				}
				elseif($PGQBList[$PGid]==3){
					$ItemPriceFlag[$pid] = 3;
				}
				else{
					$ItemPriceFlag[$pid] = 0;
				}
				$i = $i+1;
				
				vmdebug('PGItemQBPrice:', $PGItemQBPrice);
			}


			//$SIProduct is to store the Best Product Level Sale Item Price for each product in cart
			$SIProduct=Array();
			$i=0;
			
			//Loop through each product and get its best sale price
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				//$pid = '__'.$i.'__'.$product_id;
				$pid = '__'.$i.'__'.$product_id.'__'.$tmpProduct->purchase_type;
				
				$price_type=null;

				if(isset($tmpProduct->purchase_type))
					$price_type=$tmpProduct->purchase_type;
				/*else
					$price_type='1';*/

				//Get custom fields data and loop through to note the chop, join and QB pricing details
				foreach($tmpProduct->customfields as $customfield){
					if($customfield->virtuemart_custom_id==17) {
						$chop_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==18) {
						$join_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==19) {
						$sheet_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==20) {
						$qb1_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==21) {
						$qb2_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==22) {
						$qb3_price = $customfield->customfield_value; 
					}
				}			
				$product_price = $tmpProduct->prices['product_price'];	
				
				//Get all sale items for current product
				$productM =VmModel::getModel('product');
				$si = $productM->getSI($product_id);
				//vmdebug('si',$si);
				if(count($si)>0){
					$saleitem_id = $si[0];

					$minprice=0; //This holds the best price for curent sale item

					//Loop through each sale item to get best price
					foreach($si as $saleitem_id){
						//Get sale item properties
						$saleitemprice=0;
						$saleitemM =VmModel::getModel('saleitem');
						$saleitem = $saleitemM->getSaleItemDetails($saleitem_id);

						$qbdiscount=$saleitem->qbdiscount;
						$SIStartDt = $saleitem->startdate;
						$SIEndDt = $saleitem->enddate;
						$SIQuantity = $saleitem->quantity;
						$todays_date = date("Y-m-d");
						$unitofmeasure = $saleitem->unitofmeasure;
						
						if($unitofmeasure=='Chop' || $unitofmeasure=='Join'){
							$ProductQuantity = $tmpProduct->quantity;
						}
						else{
							$ProductQuantity = self::$_cart->getActualQuantity($tmpProduct);
						}			
						
						$currentunit = '1';

						if($unitofmeasure=="Chop"){
							$currentunit = '2';
						}
						elseif($unitofmeasure=="Join"){
							$currentunit = '3';
						}
						
						/*vmdebug('$saleitem',$saleitem);
						vmdebug('$ProductQuantity',$ProductQuantity);
						vmdebug('$currentunit',$currentunit);
						vmdebug('$price_type',$price_type);
						vmdebug('$SIStartDt',$SIStartDt);
						vmdebug('$SIEndDt',$SIEndDt);
						vmdebug('flatrate',$saleitem->flatrate);
						vmdebug('percentage',$saleitem->percentage);
						vmdebug('qb',$saleitem->qb);*/

						//Check if effective dates and quantity are specified, if so validate them
						//Nanda - Need to include unit of measure and if that doesn't match, need to return base price
						if( (($SIStartDt==null) ||  ($SIStartDt=='0000-00-00') || ($SIStartDt=='') || ($SIStartDt<=$todays_date)) && (($SIEndDt==null) ||  ($SIEndDt=='0000-00-00') || ($SIEndDt=='') || ($SIEndDt>=$todays_date)) && (($SIQuantity==null)|| ($SIQuantity=='') || ($ProductQuantity >= $SIQuantity || $is_PC1 )) && ($price_type==null || $currentunit == $price_type)){
													  
							if($saleitem->flatrate) //If saleitem has flat rate defined use it
							{
								$saleitemprice=$saleitem->flatrate;
							}
							else if($saleitem->percentage) //If saleitem has percentage, get the discount
							{
								$saleitemper = $saleitem->percentage;
								
								if($price_type==1){
									$temp=$product_price;
									
									if($group_name == "Artist"){
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								else if($price_type==2){
									$temp=$chop_price;
									
									if($group_name == "Artist"){
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								else if($price_type==3){
									$temp=$join_price;
									
									if($group_name == "Artist"){
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								else if($price_type==4){
									$temp=$sheet_price;
									
									if($group_name == "Artist"){
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								$saleitemprice = $temp - ($temp*$saleitemper/100);
							}
							elseif($saleitem->qb) //If saleitem has QBs defined, get the corresponding QB Price
							{
								$QBFlag = $saleitem->qb;
								
								if($QBFlag =='-100'){
									$saleitemprice=$qb1_price;
									$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
								}
								elseif($QBFlag =='-200'){
									$saleitemprice=$qb2_price;
									$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
								}
								elseif($QBFlag =='-300'){
									$saleitemprice=$qb3_price;
									$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
								}
							}
							//For first item, assign the saleitem price, and compare it from next time for best price
							if(($minprice==0 || $saleitemprice<$minprice)&&($saleitemprice!=0)){
								$minprice=$saleitemprice;
							}
							
						}			
					}
					//vmdebug('$minprice',$minprice);
					//vmdebug('$saleitemprice',$saleitemprice);

					//If no saleitem applicable for current product, get its base price as min price
					if($minprice==0){

						if($price_type==1){
							$minprice=$product_price;
							
							if($group_name == "Artist") {
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}
						else if($price_type==2){
							$minprice=$chop_price;

							if($group_name == "Artist"){
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}
						else if($price_type==3){
							$minprice=$join_price;

							if($group_name == "Artist"){
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}
						else if($price_type==4){ 
							$minprice=$sheet_price;

							if($group_name == "Artist"){
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}			
					}
					$SIProduct[$pid]=$minprice;
				}
				
				$i=$i+1;
			}


			//Nanda Begin - Price Class Level Sale Discounts Code
			
			$PCList=Array();
			//Introducing new array to hold only number of chops or joins (without multiplying the feet)
			//to get correct sale items applied
			$PCQuantityList=Array();
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
			
				$price_type=null;

				if(isset($tmpProduct->purchase_type))
					$price_type=$tmpProduct->purchase_type;
				/*else
					$price_type='1';		*/

				//Get all sale items for current product
				$productM =VmModel::getModel('product');
				$pc = $productM->getPC($product_id);
				
				if(count($pc)>0){
					$PCid=$pc[0];
					if(isset ($PCList[$PCid])){
						$PCList[$PCid] = $PCList[$PCid] + self::$_cart->getActualQuantity($tmpProduct);
					}
					else{
						$PCList[$PCid]=	self::$_cart->getActualQuantity($tmpProduct);
					}

					//Store only number of chops/joins 
					if(isset ($PCQuantityList[$PCid])){
						$PCQuantityList[$PCid]=$PCQuantityList[$PCid]+ $tmpProduct->quantity;
					}
					else{
						$PCQuantityList[$PCid]=	$tmpProduct->quantity;
					}		
				}
			}

			//$SIPriceClass is to store the Best PC Level Sale Item Price for each product in cart
			$SIPriceClass=Array();
			$SIPriceClassQ=Array(); //Nanda 2011-12
			$SIPriceClassPCQ=Array(); //Nanda 2011-12
			
			$i=0;
			//Loop through each product and get its best sale price at Price Class Level
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				//$pid = '__'.$i.'__'.$product_id;
				$pid = '__'.$i.'__'.$product_id.'__'.$tmpProduct->purchase_type;
			
				$price_type=null;

				if(isset($tmpProduct->purchase_type))
					$price_type=$tmpProduct->purchase_type;
				/*else
					$price_type='1';		*/

				//Get custom fields data and loop through to note the chop, join and QB pricing details
				foreach($tmpProduct->customfields as $customfield){
					if($customfield->virtuemart_custom_id==17) {
						$chop_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==18) {
						$join_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==19) {
						$sheet_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==20) {
						$qb1_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==21) {
						$qb2_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==22) {
						$qb3_price = $customfield->customfield_value; 
					}
				}			
				$product_price = $tmpProduct->prices['product_price'];				
					
				//Get all sale items for current product
				$productM =VmModel::getModel('product');
				$pc = $productM->getPC($product_id);
				
				if(count($pc)>0){
					$PCid=$pc[0];
			
					$priceclassM =VmModel::getModel('priceclass');
					$salieitems = $priceclassM->getSI($PCid);

					$minprice=0; //This holds the best price for curent sale item
			  
					//Loop through each sale item to get best price
					
					$minprice=0; //This holds the best price for curent sale item
					//vmdebug('TEST1 ', $salieitems);
					//Loop through each sale item to get best price
					//foreach($si as $saleitem_id){
					foreach($salieitems as $saleitem_id){
					//vmdebug('TEST2 ');
						//Get sale item properties
						$saleitemprice=0;
						$saleitemM =VmModel::getModel('saleitem');
						$saleitem = $saleitemM->getSaleItemDetails($saleitem_id);

						$qbdiscount=$saleitem->qbdiscount;
						$SIStartDt = $saleitem->startdate;
						$SIEndDt = $saleitem->enddate;
						$SIQuantity = $saleitem->quantity;
						$todays_date = date("Y-m-d");
						$unitofmeasure = $saleitem->unitofmeasure;
						/*vmdebug('TEST111 $saleitem->qbdiscount ', $saleitem->qbdiscount);
						vmdebug('TEST111 $saleitem->startdate ', $saleitem->startdate);
						vmdebug('TEST111 $saleitem->enddate ', $saleitem->enddate);
						vmdebug('TEST111 $saleitem->quantity ', $saleitem->quantity);
						vmdebug('TEST111 $saleitem->unitofmeasure ', $saleitem->unitofmeasure);*/

						if($unitofmeasure=='Chop' || $unitofmeasure=='Join'){
							$PCQuantity = $PCQuantityList[$PCid];
						}
						else{
							$PCQuantity = $PCList[$PCid];
						}			
						
						$qbdiscount = $saleitem->qbdiscount; 
						$currentunit = '1';
									
						if($unitofmeasure=="Chop"){
							$currentunit = '2';
						}
						elseif($unitofmeasure=="Join"){
							$currentunit = '3';
						}

						//vmdebug('TEST111 $currentunit ', $currentunit);
						//vmdebug('TEST111 $PCid ', $PCid);
						//vmdebug('TEST111 $PCQuantity ', $PCQuantity);
						//vmdebug('TEST111 $PCQuantityList ', $PCQuantityList);
						//vmdebug('TEST3 ');
						//vmdebug('TEST111 $todays_date ', $todays_date);
						//vmdebug('TESSSST::: ',(($SIStartDt==null) || ($SIStartDt=='0000-00-00') || ($SIStartDt=='') || ($SIStartDt<=$todays_date)) && (($SIEndDt==null) || ($SIEndDt=='0000-00-00') || ($SIEndDt=='') || ($SIEndDt>=$todays_date)) && (($SIQuantity==null)|| ($SIQuantity=='') ||($PCQuantity >= $SIQuantity)));
						//vmdebug('TEST111 $saleitem->percentage ', $saleitem->percentage);
						//vmdebug('TEST111 $saleitem->qb ', $saleitem->qb);



						//Check if effective dates and quantity are specified, if so validate them
						//Nanda - Need to include unit of measure and if that doesn't match, need to return base price
						if( (($SIStartDt==null) || ($SIStartDt=='0000-00-00') || ($SIStartDt=='') || ($SIStartDt<=$todays_date)) && (($SIEndDt==null) || ($SIEndDt=='0000-00-00') || ($SIEndDt=='') || ($SIEndDt>=$todays_date)) && (($SIQuantity==null)|| ($SIQuantity=='') ||($PCQuantity >= $SIQuantity || $is_PC1))&& ( $price_type==null || $currentunit == $price_type)){
						 
							if(strtoupper($saleitem->discounttype)=='FLATRATE' && $saleitem->flatrate) //If saleitem has flat rate defined use it
							{
								$saleitemprice=$saleitem->flatrate;
							}
							else if(strtoupper($saleitem->discounttype)=='PERCENTAGE' && $saleitem->percentage) //If saleitem has percentage, get the discount
							{
								$temp1=$saleitem->percentage;

								if($price_type==1){
									$temp=$product_price;

									if($group_name == "Artist")
									{
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								else if($price_type==2){
									$temp=$chop_price;
									if($group_name == "Artist")
									{
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								else if($price_type==3){
									$temp=$join_price;

									if($group_name == "Artist")
									{
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								else if($price_type==4){
									$temp=$sheet_price;

									if($group_name == "Artist"){
										$temp= $temp+ ($temp*$artist/100);
									}
								}
								
								$saleitemprice = $temp - ($temp*$temp1/100);
							}
							elseif(strtoupper($saleitem->discounttype)=='QB' && $saleitem->qb) //If saleitem has QBs defined, get the corresponding QB Price
							{
								
								$QBFlag = $saleitem->qb;

								if($QBFlag =='-100')
								{
									$saleitemprice=$qb1_price;
									$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
								}
								elseif($QBFlag =='-200')
								{
									$saleitemprice=$qb2_price;
									$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100); 
								}
								elseif($QBFlag =='-300')
								{
									$saleitemprice=$qb3_price;
									$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100); 
								}
							}

							//For first item, assign the saleitem price, and compare it from next time for best price
							if(($minprice==0 || $saleitemprice<$minprice)&&($saleitemprice!=0)){
								$minprice=$saleitemprice;
								$SIPriceClassSIQ[$pid]= $SIQuantity;
								$SIPriceClassPCQ[$pid]= $PCQuantity;
							}
						}
					}
					/*vmdebug("pid:",$pid);
					vmdebug("minprice:",$minprice);
					vmdebug("price_type:",$price_type);
					vmdebug("product_price:",$product_price);
					vmdebug("chop_price:",$chop_price);
					vmdebug("join_price:",$join_price);
					vmdebug("sheet_price:",$sheet_price);
					*/
					//If no saleitem applicable for current product, get its base price as min price
					if($minprice==0){
						if($price_type==1){
							$minprice=$product_price;
							
							if($group_name == "Artist")
							{
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}
						else if($price_type==2){
							$minprice=$chop_price;

							if($group_name == "Artist")
							{
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}
						else if($price_type==3){
							$minprice=$join_price;
							
							if($group_name == "Artist")
							{
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						}
						else if($price_type==4){
							$minprice=$sheet_price;
							if($group_name == "Artist")
							{
								$minprice= $minprice+ ($minprice*$artist/100);
							}
						} else {
							$minprice=$product_price;
							
							if($group_name == "Artist")
							{
								$minprice= $minprice+ ($minprice*$artist/100);
							}							
						}						
					}
					
					$SIPriceClass[$pid]=$minprice;
				}
				$i=$i+1;
			}	
			
			
			
			
			
			
			
			//Nanda Begin - Price Group Level Sale Discounts Code
			
			//$PGList
			
			//$SIPriceGroup is to store the Best PG Level Sale Item Price for each product in cart
			$SIPriceGroup=Array();
			$SIPriceGroupQ=Array(); //Nanda 2011-12
			$SIPriceGroupPGQ=Array(); //Nanda 2011-12
			
			$i=0;
			//Loop through each product and get its best sale price at Price Group Level
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				//$pid = '__'.$i.'__'.$$product_id;
				$pid = '__'.$i.'__'.$product_id.'__'.$tmpProduct->purchase_type;
			
				$price_type=null;

				if(isset($tmpProduct->purchase_type))
					$price_type=$tmpProduct->purchase_type;
				/*else
					$price_type='1';		*/

				//Get custom fields data and loop through to note the chop, join and QB pricing details
				foreach($tmpProduct->customfields as $customfield){
					if($customfield->virtuemart_custom_id==10) {
						$uom = $customfield->customfield_value; 
					} else if($customfield->virtuemart_custom_id==17) {
						$chop_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==18) {
						$join_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==19) {
						$sheet_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==20) {
						$qb1_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==21) {
						$qb2_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==22) {
						$qb3_price = $customfield->customfield_value; 
					}
				}			
				$product_price = $tmpProduct->prices['product_price'];				
				//Get price class id of current product
				$productM =VmModel::getModel('product');
				$pc = $productM->getPC($product_id);
				
				if(count($pc)>0){
					$PCid=$pc[0];

					//Get price group id for current product based on its price class
					$priceclassM =VmModel::getModel('priceclass');
					$pg = $priceclassM->getPG($PCid);
					
					if(count($pg)>0){
						$PGid=$pg[0];

						//Get all sale items for current price group
						$pricegroupM =VmModel::getModel('pricegroup');
						$si = $pricegroupM->getSI($PGid);
					  
						$minprice=0; //This holds the best price for curent sale item

						//Loop through each sale item to get best price
						
						foreach($si as $saleitem_id){
							//Get sale item properties
							$saleitemprice=0;
							$saleitemM =VmModel::getModel('saleitem');
							$saleitem = $saleitemM->getSaleItemDetails($saleitem_id);

							$qbdiscount=$saleitem->qbdiscount;
							$SIStartDt = $saleitem->startdate;
							$SIEndDt = $saleitem->enddate;
							$SIQuantity = $saleitem->quantity;
							$todays_date = date("Y-m-d");
							$unitofmeasure = $saleitem->unitofmeasure;
							$PGQuantity = $PGList[$PGid];
							
							$currentunit = '1';
							
							if($unitofmeasure=="Chop"){
								$currentunit = '2';
							}
							elseif($unitofmeasure=="Join"){
								$currentunit = '3';
							}

							if($product->virtuemart_category_id=='3'||$product->virtuemart_category_id=='4'||$product->virtuemart_category_id=='5')
							{

								if($uom=="Pallet" )
									$price_type='5';
								elseif($uom=="Case" )
									$price_type='6';
								elseif($uom=="Sheets" )
									$price_type='7';
								elseif($uom=="Sq Foot" )
									$price_type='8';
									
								if($unitofmeasure=="Case"){
									$currentunit = '6';
								}
								elseif($unitofmeasure=="Sheets"){
									$currentunit = '7';
								}
								elseif($unitofmeasure=="Sq Foot"){
									$currentunit = '8';
								}
								else{
									$currentunit = '5';
								}
							}
							//Check if effective dates and quantity are specified, if so validate them
							//Nanda - Need to include unit of measure and if that doesn't match, need to return base price
							if( (($SIStartDt==null) ||  ($SIStartDt=='0000-00-00') || ($SIStartDt=='') || ($SIStartDt<=$todays_date)) && (($SIEndDt==null) ||  ($SIEndDt=='0000-00-00') || ($SIEndDt=='') || ($SIEndDt>=$todays_date)) && (($SIQuantity==null)|| ($SIQuantity=='') || ($PGQuantity >= $SIQuantity))&& ($currentunit == $price_type))
							{
							 
								if($saleitem->flatrate) //If saleitem has flat rate defined use it
								{
									$saleitemprice=$saleitem->flatrate;
								}
								else if($saleitem->percentage) //If saleitem has percentage, get the discount
								{
									if($price_type==1){
										$temp=$product_price;

										if($group_name == "Artist")
										{
											$temp= $temp+ ($temp*$artist/100);
										}
									}
									else if($price_type==2){
										$temp=$chop_price;

										if($group_name == "Artist")
										{
											$temp= $temp+ ($temp*$artist/100);
										}
									}
									else if($price_type==3){
										$temp=$join_price;

										if($group_name == "Artist")
										{
											$temp= $temp+ ($temp*$artist/100);
										}
									}
									else if($price_type==4){
										$temp=$sheet_price;
										
										if($group_name == "Artist")
										{
											$temp= $temp+ ($temp*$artist/100);
										}
									}
									else{
										$temp=$product_price;

										if($group_name == "Artist")
										{
											$temp= $temp+ ($temp*$artist/100);
										}
									}
									
									$temp1=$saleitem->percentage;
									$saleitemprice= $temp - ($temp*$temp1/100);
								}
								elseif($saleitem->qb) //If saleitem has QBs defined, get the corresponding QB Price
								{
									$QBFlag = $saleitem->qb;
															
									if($QBFlag =='-100'){
										$saleitemprice=$qb1_price;
										$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
									}
									elseif($QBFlag =='-200'){
										$saleitemprice=$qb2_price;
										$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
									}
									elseif($QBFlag =='-300'){
										$saleitemprice=$qb3_price;
										$saleitemprice=$saleitemprice-($saleitemprice* $qbdiscount/100);
									}
									
								}
								//For first item, assign the saleitem price, and compare it from next time for best price
								if(($minprice==0 || $saleitemprice<$minprice)&&($saleitemprice!=0)){
									$minprice=$saleitemprice;
									
									$SIPriceGroupSIQ[$pid]= $SIQuantity;
									$SIPriceGroupPGQ[$pid]= $PGQuantity;
								}
							}
						}

						//If no saleitem applicable for current product, get its base price as min price
						if($minprice==0){
							$minprice=$product_price;
							
							if($price_type==1){
								$minprice=$product_price;

								if($group_name == "Artist")
								{
									$minprice= $minprice+ ($minprice*$artist/100);
								}
							}
							else if($price_type==2){
								$minprice=$chop_price;

								if($group_name == "Artist")
								{
									$minprice= $minprice+ ($minprice*$artist/100);
								}
							}
							else if($price_type==3){
								$minprice=$join_price;

								if($group_name == "Artist")
								{
									$minprice= $minprice+ ($minprice*$artist/100);
								}
							}
							else if($price_type==4){
								$minprice=$sheet_price;

								if($group_name == "Artist")
								{
									$minprice= $minprice+ ($minprice*$artist/100);
								}
							}			
						}
						
						$SIPriceGroup[$pid]=$minprice;
					}
				}
				$i=$i+1;
			}
			
			//Nanda Begin Calculate Best of 3 levels of sale item and QB
			
			$ProductBestPrice=Array();

			$i=0;
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				//$pid = '__'.$i.'__'.$product_id;
				$pid = '__'.$i.'__'.$product_id.'__'.$tmpProduct->purchase_type;
				
				$BestSalePrice = null;

				if((($SIPriceClassSIQ[$pid]==null)|| ($SIPriceClassPCQ[$pid]==null) || ($SIPriceClassPCQ[$pid] >= $SIPriceClassSIQ[$pid]))){
					$BestSalePrice = $SIProduct[$pid];
					if(($BestSalePrice==null) || (($SIPriceClass[$pid]!=null) &&($BestSalePrice > $SIPriceClass[$pid]))){
						$BestSalePrice = $SIPriceClass[$pid];
					}
				}

				if((($SIPriceGroupSIQ[$pid]==null)||($SIPriceGroupPGQ[$pid] >= $SIPriceGroupSIQ[$pid]))){
					if($BestSalePrice==null){
						$BestSalePrice = $SIProduct[$pid];
					}
					
					if(($BestSalePrice==null) || (($SIPriceGroup[$pid]!=null)&&($BestSalePrice > $SIPriceGroup[$pid]))){
						$BestSalePrice = $SIPriceGroup[$pid];
					}
				}		
						
				
				if(($BestSalePrice==null) || ($BestSalePrice >= $PGItemQBPrice[$pid])){
					$ProductBestPrice[$pid] = $PGItemQBPrice[$pid];
				}
				else{
					$ProductBestPrice[$pid] = $BestSalePrice;
					$ItemPriceFlag[$pid] = 4; 
				}
				 $i=$i+1;
			}	
				
			//Nanda
			/*vmdebug('IPriceClassSIQ:',$SIPriceClassSIQ);
			vmdebug('SIPriceGroupSIQ:',$SIPriceGroupSIQ);
			vmdebug('SIPriceClassPCQ:',$SIPriceClassPCQ);
			vmdebug('SIPriceGroupPGQ:',$SIPriceGroupPGQ);
			vmdebug('SIPriceGroup:',$SIPriceGroup);
			vmdebug('SIPriceClass:',$SIPriceClass);
			vmdebug('SIProduct:',$SIProduct);
			vmdebug('PGItemQBPrice:',$PGItemQBPrice);
			vmdebug('BestSalePrice:',$BestSalePrice);
			vmdebug('ProductBestPrice:',$ProductBestPrice);
			vmdebug('ItemPriceFlag:',$ItemPriceFlag);
*/

			$i=0;
			//Loop through each product and get its best sale price at Price Group Level
			foreach (self::$_cart->products as $tmpProduct) {
				$product_id= $tmpProduct->virtuemart_product_id;
				//$pid = '__'.$i.'__'.$product_id;
				$pid = '__'.$i.'__'.$product_id.'__'.$tmpProduct->purchase_type;
			
				$price_type=null;

				if(isset($tmpProduct->purchase_type))
					$price_type=$tmpProduct->purchase_type;
				/*else
					$price_type='1';		*/

				//Get custom fields data and loop through to note the chop, join and QB pricing details
				foreach($tmpProduct->customfields as $customfield){
					if($customfield->virtuemart_custom_id==10) {
						$uom = $customfield->customfield_value; 
					} else if($customfield->virtuemart_custom_id==17) {
						$chop_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==18) {
						$join_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==19) {
						$sheet_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==20) {
						$qb1_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==21) {
						$qb2_price = $customfield->customfield_value; 
					} else if ($customfield->virtuemart_custom_id==22) {
						$qb3_price = $customfield->customfield_value; 
					}
				}			
				$product_price = $tmpProduct->prices['product_price'];
				//vmdebug('VVproduct_price 00:',$product_price);
				//vmdebug('VVprice_type:',$price_type);
			
				$dimcal = self::$_cart->getActualQuantity($tmpProduct);
				//vmdebug('VVproduct_price 00 1:',$product_price);
				if($price_type=='1')
				{		
					if($group_name == "Artist"){
						$product_price= $product_price+ ($product_price*$artist/100);
					}
				}
				else if($price_type=='2')
				{
					$product_price=$chop_price;

					if($group_name == "Artist"){
						$product_price= $product_price+ ($product_price*$artist/100);
					}
				}
				else if($price_type=='3')
				{
					$product_price=$join_price;
					if($group_name == "Artist"){
						$product_price= $product_price+ ($product_price*$artist/100);
					}
				}
				else if($price_type=='4')
				{
					$product_price=$sheet_price;
					if($group_name == "Artist"){
						$product_price= $product_price+ ($product_price*$artist/100);
					}
				} else {
					if($group_name == "Artist"){
						$product_price= $product_price+ ($product_price*$artist/100);
					}					
				}
				$product_baseprice = round( $product_price, 2 );
				//vmdebug('VVproduct_price 00 2:',$product_price);
				//vmdebug('VMproduct_baseprice:', $product_baseprice);
				//vmdebug('VMProductBestPrice:', $ProductBestPrice);
				
				if($group_name == "Wholesale"){
					$product_price = round( $ProductBestPrice[$pid], 2 );
					if($product_baseprice==$ProductBestPrice[$pid]){
							$ItemPriceFlag[$pid] = 0;
					}
				}
				else{
					$ItemPriceFlag[$pid] = 0;
				}
				/*vmdebug('VMgroup_name:', $group_name);
				vmdebug('VMdimcal:', $dimcal);
				vmdebug('VMproduct_price:', $product_price);
				vmdebug('VMproduct_baseprice:', $product_baseprice);*/
				//vmdebug('VVproduct_price 00 3:',$product_price);
				if($group_name == "Artist"){
					$subtotal = $product_baseprice * $dimcal;
				}else{
					$subtotal = $product_price * $dimcal;
				}
				//vmdebug('VVproduct_price 00 4:',$product_price);
				$tmpProduct->custom_subtotal = $subtotal;
				$tmpProduct->custom_dimcal = $dimcal/$tmpProduct->quantity;
				$tmpProduct->custom_product_baseprice = $product_baseprice;
				$tmpProduct->custom_product_price = $product_price;
				$tmpProduct->custom_product_price_flag = $ItemPriceFlag[$pid];
				//vmdebug('VVproduct_price 11:',$product_price);
				$UM = '';
				if($price_type=='1'){
					if($tmpProduct->virtuemart_category_id=='11')
					   $UM='Case';  
					else
						$UM='Length';
				}
				else if($price_type=='2'){
					$UM='Chop';
				}
				else if($price_type=='3'){
					$UM='Join';
				}
				else if($price_type=='4'){
					$UM='Sheet';
				}
				else{
					if($tmpProduct->virtuemart_category_id=='11')
						$UM='Case';   
				}		
				if($UM=='')
					$UM=$uom;
				
				$tmpProduct->custom_uom = $UM;

				
				//NANDA
				//Code to display additional attributes info after product name in the cart 
				$custom_name_suffix='';
				if($tmpProduct->virtuemart_category_id=='7'){
					if ($tmpProduct->base2!=0) {
						$UM = 'Chop';
					}
					else{
						$UM = 'Length';
					}     
				}

				if(isset($UM) && ($UM!='')){
					$custom_name_suffix = $custom_name_suffix . $UM ;
					if( $tmpProduct->virtuemart_category_id ==1||$tmpProduct->virtuemart_category_id=='7')
						$custom_name_suffix = $custom_name_suffix . ':  ';
				}
				if($tmpProduct->virtuemart_category_id=='1' || $tmpProduct->virtuemart_category_id=='7'){
					$custom_name_suffix = $custom_name_suffix .  $tmpProduct->custom_dimcal . 'ft'.'<br>'; 
				}          
				if($tmpProduct->base1!=0){
					if(isset($UM) && ($UM=='Length'))            
						$custom_name_suffix = $custom_name_suffix . '<i>Cut to:</i>';
					else
						$custom_name_suffix = $custom_name_suffix .  '<i>Width:</i>';

					$custom_name_suffix = $custom_name_suffix . $tmpProduct->base1 . '  ';
					if($tmpProduct->fraction1!=0) 
						$custom_name_suffix = $custom_name_suffix . $tmpProduct->fraction1. '  ';
				}

				if ($tmpProduct->base2!=0) {
					if(isset($UM) && ($UM!='Length'))            
						$custom_name_suffix = $custom_name_suffix . '<i>Length:</i>'. '  ';

					$custom_name_suffix = $custom_name_suffix . $tmpProduct->base2 . '  ';  
					if($tmpProduct->fraction2!=0) 
						$custom_name_suffix = $custom_name_suffix . $tmpProduct->fraction2. '  ';
				}
				
				$tmpProduct->custom_name_suffix = $custom_name_suffix;
				
				//Nanda
				//$tmpProduct->prices["product_price"] = $product_price;
				
				$i=$i+1;
			}
		}
		//vmdebug('VVproduct_price 22:',$product_price);
		$cart_total_price=0;
		foreach (self::$_cart->products as $tmpProduct) {	
//vmdebug('VM00:', $tmpProduct);		
			$cart_total_price = $cart_total_price + $tmpProduct->custom_subtotal;
			$tmpProduct->prices["discountedPriceWithoutTax"] = $tmpProduct->custom_subtotal;
		}
		self::$_cart->cart_total_price = $cart_total_price;
		
		//vmdebug('self::$_cart:',self::$_cart);
		//vmdebug('VVproduct_price 33:',$product_price);
		//NANDA
		//vmdebug("CART NEW:",$cartSession);
		//vmdebug('Cart123:',self::$_cart->products[0]);
		return self::$_cart;
	}


	function loadSetRenderBTSTAddress(){

		//If the user is logged in and exists, we check if he has already addresses stored
		if(!empty($this->user->virtuemart_user_id)){

			foreach ($this->user->userInfo as $address) {
				if ($address->address_type == 'BT') {
					$this->saveAddressInCart((array) $address, $address->address_type,false);
				} else {
					if(!empty($this->selected_shipto) and $address->virtuemart_userinfo_id==$this->selected_shipto){
						$this->saveAddressInCart((array) $address, $address->address_type,false,'');
					}
				}
			}
			if(empty($this->selected_shipto)){
				$this->STsameAsBT = 1;
				$this->ST = 0;
			}
		}
	}


	//function prepareAddressDataInCart($type='BT',$new = false,$virtuemart_user_id = null){
	function prepareAddressFieldsInCart(){

		$userFieldsModel =VmModel::getModel('Userfields');

		$types = array('BT','ST');
		foreach($types as $type){
			if($type=='ST'){
				$preFix = 'shipto_';
			} else {
				$preFix = '';
			}

			$addresstype = $type.'address'; //for example BTaddress
			$userFields = $userFieldsModel->getUserFieldsFor('cart',$type);
			$this->$addresstype = $userFieldsModel->getUserFieldsFilled(
				$userFields
				,$this->$type
				,$preFix
			);
		}

	}

	/**
	 * @author Max Milbers
	 */
	public function loadCart(&$existingSession){
		$currentUser = JFactory::getUser();
		if(!$currentUser->guest and $existingSession){
			$model = new VmModel();
			$carts = $model->getTable('carts');
			if(!empty($existingSession->virtuemart_cart_id)){
				$carts->load($existingSession->virtuemart_cart_id,'virtuemart_cart_id');
			} else {
				$carts->load($currentUser->id,0,' ORDER BY `modified_on` DESC');
			}

			$cartData = $carts->loadFieldValues();
			unset($cartData['_inCheckOut']);
			unset($cartData['_dataValidated']);
			unset($cartData['_confirmDone']);
			unset($cartData['_fromCart']);
			$this->virtuemart_cart_id = $cartData['virtuemart_cart_id'];
			if($cartData and !empty($cartData['cartData'])){
				$cartData['cartData'] = (object)json_decode($cartData['cartData'],true);

				if(!empty($cartData['cartData'])){
					if(!empty($cartData['cartData']->cartProductsData) and is_array($cartData['cartData']->cartProductsData)){
						foreach($cartData['cartData']->cartProductsData as $k => $product){
							foreach($existingSession->cartProductsData as $kses => $productses){
								if($product==$productses){
									unset($cartData['cartData']->cartProductsData[$k]);
								}
							}
						}
					}

					foreach($cartData['cartData'] as $key=>$value){
						if(is_array($value)){
							$existingSession->$key = array_merge( $value,(array)$existingSession->$key);
						} else if(empty($existingSession->$key)){
							$existingSession->$key = $cartData['cartData']->$key;
						}
					}

					if(count($existingSession->_triesValidateCoupon)>6){
						$existingSession->_triesValidateCoupon = array_slice($existingSession->_triesValidateCoupon,0,6);
						vmdebug('Coupon were blocked, release 1');
					}
				}
			}
		}
	}

	public function storeCart($cartDataToStore = false){
		$adminID = vmAccess::getBgManagerId();
		$currentUser = JFactory::getUser();
		if(!$currentUser->guest && (!$adminID || $adminID == $currentUser->id)){
			$model = new VmModel();
			$carts = $model->getTable('carts');
			if(!$cartDataToStore) $cartDataToStore = json_encode($this->getCartDataToStore());

			$cObj = new StdClass();
			if(!empty($this->virtuemart_cart_id)) $cObj->virtuemart_cart_id = (int) $this->virtuemart_cart_id;
			$cObj->virtuemart_user_id = (int) $currentUser->id;
			$cObj->virtuemart_vendor_id = (int) $this->vendorId;
			$cObj->cartData = $cartDataToStore;
			$carts->bindChecknStore($cObj);

			if(!empty($cObj->virtuemart_cart_id)){
				$this->virtuemart_cart_id = $cObj->virtuemart_cart_id;
			}
		}
	}

	public function deleteCart(){

		$model = new VmModel();
		$carts = $model->getTable('carts');

		if(!empty($this->virtuemart_cart_id)){
			$carts->delete($this->virtuemart_cart_id,'virtuemart_cart_id');
		} else {
			$currentUser = JFactory::getUser();
			if(!empty($currentUser->id)) {
				$carts->delete($currentUser->id);
			}
		}

	}

	/**
	 * Set the cart in the session
	 *
	 * @access public
	 * @param array $cart the cart to store in the session
	 */
	public function setCartIntoSession($storeDb = false, $forceWrite = false) {

		$session = JFactory::getSession();


		$sessionCart = $this->getCartDataToStore();
		$sessionCart = json_encode($sessionCart);
		$session->set('vmcart', $sessionCart,'vm');

		if($forceWrite){
			session_write_close();
			session_start();
		}

		if($storeDb){
			$this->storeCart();
		}
	}

	public function getCartDataToStore(){
		$sessionCart = new stdClass();

		$sessionCart->cartProductsData = $this->cartProductsData;
		$sessionCart->vendorId	 					= $this->vendorId;
		$sessionCart->lastVisitedCategoryId	 		= $this->lastVisitedCategoryId;
		$sessionCart->virtuemart_shipmentmethod_id	= $this->virtuemart_shipmentmethod_id;
		$sessionCart->virtuemart_paymentmethod_id 	= $this->virtuemart_paymentmethod_id;
		$sessionCart->automaticSelectedShipment 	= $this->automaticSelectedShipment;
		$sessionCart->automaticSelectedPayment 		= $this->automaticSelectedPayment;
		$sessionCart->order_number 		            = $this->order_number;

		$sessionCart->BT 							= $this->BT;
		$sessionCart->ST 							= $this->ST;
		$sessionCart->cartfields					= $this->cartfields;

		$sessionCart->couponCode 					= $this->couponCode;
		$sessionCart->_triesValidateCoupon			= $this->_triesValidateCoupon;
		$sessionCart->order_language 				= $this->order_language;

		$sessionCart->pricesCurrency				= $this->pricesCurrency;
		$sessionCart->paymentCurrency				= $this->paymentCurrency;

		//private variables
		//We nee to store this, so that we now if a user logged in before
		$sessionCart->_guest						= JFactory::getUser()->guest;
		$sessionCart->_inCheckOut 					= $this->_inCheckOut;
		$sessionCart->_inConfirm					= $this->_inConfirm;
		$sessionCart->_dataValidated				= $this->_dataValidated;
		$sessionCart->_confirmDone					= $this->_confirmDone;
		$sessionCart->STsameAsBT					= $this->STsameAsBT;
		$sessionCart->selected_shipto 				= $this->selected_shipto;
		$sessionCart->_fromCart						= $this->_fromCart;
		$sessionCart->layout						= $this->layout;
		$sessionCart->layoutPath					= $this->layoutPath;
		$sessionCart->virtuemart_cart_id			= $this->virtuemart_cart_id;
		return $sessionCart;
	}



	/**
	 * Remove the cart from the session
	 *
	 * @author Max Milbers
	 * @access public
	 */
	public function removeCartFromSession() {
		$session = JFactory::getSession();
		$session->set('vmcart', 0, 'vm');
	}

	public function setDataValidation($valid=false) {
		$this->_dataValidated = $valid;
	}

	public function getDataValidated() {
		return $this->_dataValidated;
	}

	public function getInCheckOut() {
		return $this->_inCheckOut;
	}

	public function setOutOfCheckout(){
		$this->_inCheckOut = false;
		$this->_dataValidated = false;
		$this->_blockConfirm = true;
		$this->_redirected = true;
		$this->_redirect = false;
		$this->setCartIntoSession(false,true);
	}

	public function blockConfirm(){
		$this->_blockConfirm = true;
	}

	/**
	 * For one page checkouts, disable with this the redirects
	 * @param bool $bool
	 */
	public function setRedirectDisabled($bool = TRUE){
		$this->_redirect_disabled = $bool;
	}

	/**
	 * Add a product to the cart
	 *
	 * @author Max Milbers
	 * @access public
	 */
	public function add($virtuemart_product_ids=null,&$errorMsg='') {

		$updateSession = false;
		$post = vRequest::getRequest();

		if(empty($virtuemart_product_ids)){
			$virtuemart_product_ids = vRequest::getInt('virtuemart_product_id'); //is sanitized
		}

		if (empty($virtuemart_product_ids)) {
			vmWarn('COM_VIRTUEMART_CART_ERROR_NO_PRODUCT_IDS');
			return false;
		} else {
			if(is_array($virtuemart_product_ids)){
				$virtuemart_product_ids = array_unique($virtuemart_product_ids);
			}
		}

		//NANDA
		//vmdebug('Nanda: virtuemart_product_ids', $virtuemart_product_ids);
		
		$products = array();
		$this->_productAdded = true;
		$productModel = VmModel::getModel('product');
		$customFieldsModel = VmModel::getModel('customfields');
		//Iterate through the prod_id's and perform an add to cart for each one
		foreach ($virtuemart_product_ids as $p_key => $virtuemart_product_id) {

			$product = false;
			$updateSession = true;
			$productData = array();

			//vmdebug('Nanda: virtuemart_product_ids', $virtuemart_product_ids);
			if(empty($virtuemart_product_id)){
				vmWarn('Product could not be added with virtuemart_product_id = 0');
				return false;
			} else {
				$productData['virtuemart_product_id'] = (int)$virtuemart_product_id;
			}

			if(!empty( $post['quantity'][$p_key])){
				$productData['quantity'] = (int) $post['quantity'][$p_key];
			} else {
					continue;
			}

			//NANDA
			//vmdebug('NANDA POST:', $post);
			//vmdebug('Nanda: base1', $post['base1'][$p_key]);
			//vmdebug('Nanda: fraction1', $post['fraction1'][$p_key]);
			//vmdebug('Nanda: base2', $post['base2'][$p_key]);
			//vmdebug('Nanda: fraction2', $post['fraction2'][$p_key]);			
			//vmdebug('VM11:');
			if(!empty( $post['price'])){
				$productData['purchase_type'] = (int) $post['price'];
			}
			if(!empty( $post['base1'])){
				$productData['base1'] = (int) $post['base1'];
			}
			if(!empty( $post['fraction1'])){
				$productData['fraction1'] = $post['fraction1'];
			}			
			if(!empty( $post['base2'])){
				$productData['base2'] = (int) $post['base2'];
			}
			if(!empty( $post['fraction2'])){
				$productData['fraction2'] = $post['fraction2'];
			}			
			
			
			if(!empty( $post['customProductData'][$virtuemart_product_id])){
				//$productData['customProductData']
				$customProductData  = $post['customProductData'][$virtuemart_product_id];
			} else {
				$customProductData = array();
			}
//vmdebug('VM22:');
			//Now we check if the delivered customProductData is correct and add missing
			$productTemp = $productModel->getProduct($virtuemart_product_id, true, false,true,$productData['quantity']);
			$productTemp->modificatorSum = null;
			$product = clone($productTemp);
//vmdebug('VM33:');
			//vmdebug('VMproduct_price:',$productTemp);
			
			if(VmConfig::get('multixcart',0)=='byproduct'){
				if(empty($this->vendorId)) $this->vendorId = $product->virtuemart_vendor_id;
				if(!empty($this->vendorId) and $this->vendorId != $product->virtuemart_vendor_id){
					//Product of another vendor recognised, for now we just return false,
					//later we will create here another cart (multicart)
					return false;
				}
			}

			$product->customfields = $customFieldsModel->getCustomEmbeddedProductCustomFields($product->allIds,0,1);
			$customProductDataTmp=array();

			// Some customfields may prevent the product being added to the cart
			$customFiltered = false;

			foreach($product->customfields as $customfield){

				if(!class_exists('vmCustomPlugin')) require(JPATH_VM_PLUGINS.DS.'vmcustomplugin.php');
				JPluginHelper::importPlugin('vmcustom');
				$dispatcher = JDispatcher::getInstance();
				$addToCartReturnValues = $dispatcher->trigger('plgVmOnAddToCartFilter',array(&$product, &$customfield, &$customProductData, &$customFiltered));

				if(!$customFiltered && $customfield->is_input==1){
					if(isset($customProductData[$customfield->virtuemart_custom_id][$customfield->virtuemart_customfield_id])){

						if(is_array($customProductData[$customfield->virtuemart_custom_id][$customfield->virtuemart_customfield_id])){
							if(!class_exists('vmFilter'))require(VMPATH_ADMIN.DS.'helpers'.DS.'vmfilter.php');
							foreach($customProductData[$customfield->virtuemart_custom_id][$customfield->virtuemart_customfield_id] as $i=>$customData){

								$value = vmFilter::hl( $customData,array('deny_attribute'=>'*'));
								//to strong
								/* $value = preg_replace('@<[\/\!]*?[^<>]*?>@si','',$value);//remove all html tags  */
								//lets use instead
								$value = JComponentHelper::filterText($value);
								$value = (string)preg_replace('#on[a-z](.+?)\)#si','',$value);//replace start of script onclick() onload()...
								$value = trim(str_replace('"', ' ', $value),"'") ;
								$customProductData[$customfield->virtuemart_custom_id][$customfield->virtuemart_customfield_id][$i] = (string)preg_replace('#^\'#si','',$value);
							}
						}
						if(!isset($customProductDataTmp[$customfield->virtuemart_custom_id])) $customProductDataTmp[$customfield->virtuemart_custom_id] = array();
						$customProductDataTmp[$customfield->virtuemart_custom_id][$customfield->virtuemart_customfield_id] = $customProductData[$customfield->virtuemart_custom_id][$customfield->virtuemart_customfield_id];
					}
					else if(isset($customProductData[$customfield->virtuemart_custom_id])) {
						$customProductDataTmp[$customfield->virtuemart_custom_id] = $customProductData[$customfield->virtuemart_custom_id];
						//vmdebug('my customp product data ',$customProductData[$customfield->virtuemart_custom_id]);
					}
				} else {
					if(!isset($customProductDataTmp[$customfield->virtuemart_custom_id])){
						$customProductDataTmp[$customfield->virtuemart_custom_id] = array();
					} else if(!is_array($customProductDataTmp[$customfield->virtuemart_custom_id])){
						$customProductDataTmp[$customfield->virtuemart_custom_id] = array($customProductDataTmp[$customfield->virtuemart_custom_id]);
					}
					$customProductDataTmp[$customfield->virtuemart_custom_id][(int)$customfield->virtuemart_customfield_id] = false;
				}

			}

			$productData['customProductData'] = $customProductDataTmp;


			$unsetA = array();
			$found = false;


			//Now lets check if there is already a product stored with the same id, if yes, increase quantity and recalculate
			foreach($this->cartProductsData as $k => &$cartProductData){
				$cartProductData = (array)$cartProductData;
				if(empty($cartProductData['virtuemart_product_id'])){
					$unsetA[] = $k;
					$errorMsg = true;
				} else {
					if($cartProductData['virtuemart_product_id'] == $productData['virtuemart_product_id']){

						//Okey, the id is already the same, so lets check the customProductData
						$diff = !$this->deepCompare($cartProductData['customProductData'],$productData['customProductData']);
						
						if(!$diff && isset($cartProductData[purchase_type]) && isset($productData[purchase_type])){
							$diff = !($cartProductData[purchase_type] == $productData[purchase_type]);
						}
						
						if(!$diff) {												
							if(!empty( $productData['base1']) || !empty( $cartProductData['base1'])){
								$diff = !($cartProductData['base1'] == $productData['base1']);
							}
						}
						if(!$diff) {												
							if(!empty( $productData['fraction1']) || !empty( $cartProductData['fraction1'])){
								$diff = !($cartProductData['fraction1'] == $productData['fraction1']);
							}
						}
						if(!$diff) {												
							if(!empty( $productData['base2']) || !empty( $cartProductData['base2'])){
								$diff = !($cartProductData['base2'] == $productData['base2']);
							}
						}
						if(!$diff) {												
							if(!empty( $productData['fraction2']) || !empty( $cartProductData['fraction2'])){
								$diff = !($cartProductData['fraction2'] == $productData['fraction2']);
							}
						}
						
						//vmdebug('11 productData:',$productData);
						//vmdebug('11 cartProductData:',$cartProductData);
						
						if(!$diff && $productData[virtuemart_category_id]!=1 && $productData[virtuemart_category_id]!=7){

							$newTotal = $cartProductData['quantity'] + $productData['quantity'];

							if(!$product)$product = $this->getProduct((int) $productData['virtuemart_product_id'],$cartProductData['quantity']);
							if(empty($product->virtuemart_product_id)){
								vmWarn('COM_VIRTUEMART_PRODUCT_NOT_FOUND');
								$unsetA[] = $k;

							} else {
								$this->checkForQuantities($product, $newTotal);
								vmdebug("add to cart did checkForQuantities",$newTotal,$cartProductData['quantity'],$productData['quantity']);

								$product->quantity = $newTotal - $cartProductData['quantity'];
								$cartProductData['quantity'] = $newTotal;

								vmdebug('add to cart did $product->quantityAdded  ',$cartProductData['quantity']);
							}
							$found = TRUE;
							break;
						} else {
							vmdebug('product variant is different, I add to cart');
						}
					}
				}

				//add products to remove to array
				if($cartProductData['quantity']==0){
					$unsetA[] = $k;
				}

			}

			if(!$found){
				if(!$product)$product = $this->getProduct( (int)$productData['virtuemart_product_id'],$productData['quantity']);
				if(!empty($product->virtuemart_product_id)){

					$this->checkForQuantities($product, $product->quantity);
					vmdebug('my $productData $productData ',$productData);
					if(!empty($product->quantity)){
						$productData['quantity'] = $product->quantity;
						$this->cartProductsData[] = $productData;
					} else {
						$errorMsg = true;
					}
				} else {
					$errorMsg = true;
				}

			}

			if($product){

				//NANDA
				if(!empty( $post['price'])){
					$product->purchase_type = (int) $post['price'];
				}
				if(!empty( $post['base1'])){
					$product->base1 = (int) $post['base1'];
				}
				if(!empty( $post['fraction1'])){
					$product->fraction1 = $post['fraction1'];
				}			
				if(!empty( $post['base1'])){
					$product->base2 = (int) $post['base2'];
				}
				if(!empty( $post['fraction2'])){
					$product->fraction2 = $post['fraction2'];
				}			
			
				$products[] = $product;
			}

			//Remove the products which have quantity=0
			foreach($unsetA as $v){
				unset($this->cartProductsData[$v]);
			}
		}

		if ($updateSession== false) return false ;
		$this->_dataValidated = false;
		// End Iteration through Prod id's
		$this->setCartIntoSession(true);
		return $products;
	}

	static public function deepCompare($a,$b) {
		if(is_object($a) && is_object($b)) {
			if(get_class($a)!=get_class($b))
				return false;
			foreach($a as $key => $val) {
				if(!self::deepCompare($val,$b->$key))
					return false;
			}
			return true;
		}
		else if(is_array($a) && is_array($b)) {
			while(!is_null(key($a)) && !is_null(key($b))) {
				if (key($a)!==key($b) || !self::deepCompare(current($a),current($b)))
					return false;
				next($a); next($b);
			}
			return is_null(key($a)) && is_null(key($b));
		}
		else
			return $a===$b;
	}

	/**
	 * Remove a product from the cart
	 *
	 * @author RolandD
	 * @param array $cart_id the cart IDs to remove from the cart
	 * @access public
	 */
	public function removeProductCart($prod_id=0) {
		// Check for cart IDs
		if (empty($prod_id))
		$prod_id = vRequest::getInt('cart_virtuemart_product_id');
		unset($this->products[$prod_id]);
		if(isset($this->cartProductsData[$prod_id])){
			// hook for plugin action "remove from cart"
			if(!class_exists('vmCustomPlugin')) require(JPATH_VM_PLUGINS.DS.'vmcustomplugin.php');
			JPluginHelper::importPlugin('vmcustom');
			$dispatcher = JDispatcher::getInstance();
			$addToCartReturnValues = $dispatcher->trigger('plgVmOnRemoveFromCart',array($this,$prod_id));
			unset($this->cartProductsData[$prod_id]);
			$this->setCartIntoSession(true);
			return true;
		} else {
			vmdebug('removeProductCart $prod_id '.$prod_id,$this->cartProductsData);
			return false;
		}
	}

	/**
	 * Update a product in the cart
	 *
	 * @author Max Milbers
	 * @param array $cart_id the cart IDs to remove from the cart
	 * @access public
	 */
	public function updateProductCart() {

		$quantities = vRequest::getInt('quantity');
		if(empty($quantities)) return false;
		$updated = false;

		foreach($quantities as $key=>$quantity){
			if (isset($this->cartProductsData[$key]) and !empty($quantity) and !isset($_POST['delete_'.$key])) {
				if($quantity!=$this->cartProductsData[$key]['quantity']){
					$this->cartProductsData[$key]['quantity'] = $quantity;
					$updated = true;
				}

			} else {
				//Todo when quantity is 0,  the product should be removed, maybe necessary to gather in array and execute delete func
				unset($this->cartProductsData[$key]);
				$updated = true;
			}
		}

		$this->setCartIntoSession(true);
		return $updated;
	}


	/**
	* Get the category ID from a product ID
	*
	* @author RolandD, Patrick Kohl
	* @access public
	* @return mixed if found the category ID else null
	* @deprecated, useless function, already done in the product model
	*
	*/
	public function getCardCategoryId($virtuemart_product_id) {
		return 0;
	}

	/**
	 * Validate the coupon code. If ok,. set it in the cart
	 * @param string $coupon_code Coupon code as entered by the user
	 * @author Oscar van Eijk
	 * TODO Change the coupon total/used in DB ?
	 * @access public
	 * @return string On error the message text, otherwise an empty string
	 */
	public function setCouponCode($coupon_code) {

		if(empty($coupon_code) or $coupon_code == vmText::_('COM_VIRTUEMART_COUPON_CODE_ENTER')) {
			$this->couponCode = '';
			return false;
		} else if($this->couponCode==$coupon_code) {
			return;
		}

		if (!class_exists('CouponHelper')) {
			require(VMPATH_SITE . DS . 'helpers' . DS . 'coupon.php');
		}

		$this->prepareCartData();
		//$this->getCartPrices();

		if(!in_array($coupon_code,$this->_triesValidateCoupon)){
			$this->_triesValidateCoupon[] = $coupon_code;
		}

		if(count($this->_triesValidateCoupon)<8){
			vmdebug('setCouponCode',$coupon_code, $this->cartPrices['salesPrice']);
			$msg = CouponHelper::ValidateCouponCode($coupon_code, $this->cartPrices['salesPrice']);;
		} else{
			$msg = vmText::_('COM_VIRTUEMART_CART_COUPON_TOO_MANY_TRIES');
		}

		if (!empty($msg)) {
			$this->_dataValidated = false;
			$this->_blockConfirm = true;
			$this->getCartPrices(true);
			$this->setCartIntoSession();
			return $msg;
		}
		$this->couponCode = $coupon_code;
		//$this->getCartPrices(true);
		$this->prepareCartData(true);
		$this->setCartIntoSession(true);
		return vmText::_('COM_VIRTUEMART_CART_COUPON_VALID');
	}

	/**
	 * Check the selected shipment data and store the info in the cart
	 * @param integer $shipment_id Shipment ID taken from the form data
	 * @author Max Milbers
	 */
	public function setShipmentMethod($force=false, $redirect=true, $virtuemart_shipmentmethod_id = null) {

		if(!isset($virtuemart_shipmentmethod_id)) $virtuemart_shipmentmethod_id = vRequest::getInt('virtuemart_shipmentmethod_id', $this->virtuemart_shipmentmethod_id);
		if($this->virtuemart_shipmentmethod_id != $virtuemart_shipmentmethod_id or (!empty($virtuemart_shipmentmethod_id) and $force)){
			//$this->_dataValidated = false;
			//Now set the shipment ID into the cart
			$this->virtuemart_shipmentmethod_id = $virtuemart_shipmentmethod_id;
			if (!class_exists('vmPSPlugin')) require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
			JPluginHelper::importPlugin('vmshipment');

			//Add a hook here for other payment methods, checking the data of the choosed plugin
			$_dispatcher = JDispatcher::getInstance();
			$_retValues = $_dispatcher->trigger('plgVmOnSelectCheckShipment', array( &$this));
			$dataValid = true;
			foreach ($_retValues as $_retVal) {
				if ($_retVal === true ) {
					$this->setCartIntoSession();
					// Plugin completed successfull; nothing else to do
					break;
				} else if ($_retVal === false ) {
					if ($redirect) {
						$mainframe = JFactory::getApplication();
						$mainframe->redirect(JRoute::_('index.php?option=com_virtuemart&view=cart&task=edit_shipment',$this->useXHTML,$this->useSSL), $_retVal);
						break;
					} else {
						return;
					}
				}
			}
			$this->setCartIntoSession();
		}
	}

	public function setPaymentMethod($force=false, $redirect=true, $virtuemart_paymentmethod_id = null) {

		if(!isset($virtuemart_paymentmethod_id)) $virtuemart_paymentmethod_id = vRequest::getInt('virtuemart_paymentmethod_id', $this->virtuemart_paymentmethod_id);
		if($this->virtuemart_paymentmethod_id != $virtuemart_paymentmethod_id or (!empty($virtuemart_paymentmethod_id) and $force)){
			//$this->_dataValidated = false;
			$this->virtuemart_paymentmethod_id = $virtuemart_paymentmethod_id;
			if(!class_exists('vmPSPlugin')) require(JPATH_VM_PLUGINS.DS.'vmpsplugin.php');
			JPluginHelper::importPlugin('vmpayment');

			//Add a hook here for other payment methods, checking the data of the choosed plugin
			$msg = '';
			$_dispatcher = JDispatcher::getInstance();
			$_retValues = $_dispatcher->trigger('plgVmOnSelectCheckPayment', array( $this, &$msg));
			$dataValid = true;
			foreach ($_retValues as $_retVal) {
				if ($_retVal === true ) {
					$this->setCartIntoSession();
					// Plugin completed succesfull; nothing else to do
					break;
				} else if ($_retVal === false ) {
					if ($redirect) {
						$app = JFactory::getApplication();
						$app->redirect(JRoute::_('index.php?option=com_virtuemart&view=cart&task=editpayment',$this->useXHTML,$this->useSSL), $msg);
						break;
					} else {
						return;
					}
				}
			}
			$this->setCartIntoSession();
		}

	}

	function confirmDone() {

		//Check if data has changed meanwhile
		$cHash = $this->getCartHash();
		if($cHash != $this->_dataValidated){
			$this->_dataValidated = false;
			$app = JFactory::getApplication();
			$app->redirect(JRoute::_('index.php?option=com_virtuemart&view=cart'.$this->getLayoutUrlString(), FALSE), vmText::_('COM_VIRTUEMART_CART_CHECKOUT_DATA_CHANGED'));
		}

		//Final check
		$this->checkoutData(false);
		if ($this->_dataValidated == $cHash) {
			$this->_confirmDone = true;
			$this->orderDetails = 0;
			$this->confirmedOrder();
		} else {
			$this->_dataValidated = false;
			$this->_confirmDone = false;
			$app = JFactory::getApplication();
			$app->redirect(JRoute::_('index.php?option=com_virtuemart&view=cart'.$this->getLayoutUrlString(), FALSE), vmText::_('COM_VIRTUEMART_CART_CHECKOUT_DATA_NOT_VALID'));
		}
	}

	private function redirecter($relUrl,$redirectMsg){

		$this->_dataValidated = false;
		$app = JFactory::getApplication();
		if($this->_redirect and !$this->_redirected and !$this->_redirect_disabled){
			$this->_redirected = true;
			$this->setCartIntoSession(true);
			$app->redirect(JRoute::_($relUrl,$this->useXHTML,$this->useSSL), $redirectMsg);
			return true;
		} else {
			$this->_redirected = false;
			$this->_inCheckOut = false;
			$this->setCartIntoSession(true);
			return false;
		}
	}

	public function getLayoutUrlString(){
		$layoutName = vRequest::getCmd('layout', 'default');
		if(!empty($layoutName) and $layoutName!='default'){
			return '&layout='.$layoutName;
		} else {
			return '';
		}
	}

	public function checkoutData($redirect = true) {

		if($this->_redirected){
			$this->_redirect = false;
		} else {
			$this->_redirect = $redirect;
		}

		$layoutName = $this->getLayoutUrlString();

		$this->_inCheckOut = true;
		//This prevents that people checkout twice
		$this->setCartIntoSession(false,true);

		// Check if a minimun purchase value is set
		if (($redirectMsg = $this->checkPurchaseValue()) != null) {
			$this->_inCheckOut = false;
			return $this->redirecter('index.php?option=com_virtuemart&view=cart'.$layoutName , $redirectMsg);
		}

		$validUserDataBT = self::validateUserData();
		if ($validUserDataBT!==true) {	//Important, we can have as result -1,false and true.
			//return $this->redirecter('index.php?option=com_virtuemart&view=user&task=editaddresscart&addrtype=BT' , '');
		}

		$currentUser = JFactory::getUser();
		if(!empty($this->STsameAsBT) or (!$currentUser->guest and empty($this->selected_shipto))){	//Guest
			$this->ST = $this->BT;
		} else {
			if ($this->selected_shipto >0 ) {
				$userModel = VmModel::getModel('user');
				$stData = $userModel->getUserAddressList($currentUser->id, 'ST', $this->selected_shipto);

				if(isset($stData[0]) and is_object($stData[0])){
					$stData = get_object_vars($stData[0]);
					if($this->validateUserData('ST', $stData)>0){
						$this->ST = $stData;
					}
				} else {
					$this->selected_shipto = 0;
					$this->ST = $this->BT;
				}
			}

			//Only when there is an ST data, test if all necessary fields are filled
			$validUserDataST = self::validateUserData('ST');
			if ($validUserDataST!==true) {
				return $this->redirecter('index.php?option=com_virtuemart&view=user&task=editaddresscart&addrtype=ST' , '');
			}
		}

		$usersConfig = JComponentHelper::getParams( 'com_users' );
		$useractivation = $usersConfig->get( 'useractivation' );
		if ($currentUser ->block) {
			if($useractivation!=1){
				$redirectMsg = vmText::_('JERROR_NOLOGIN_BLOCKED');
				return $this->redirecter('index.php?option=com_virtuemart&view=user&task=editaddresscart&addrtype=BT' , $redirectMsg);
			}
		}

		if(VmConfig::get('oncheckout_only_registered',0)) {
			if(empty($currentUser->id)) {
				$redirectMsg = vmText::_('COM_VIRTUEMART_CART_ONLY_REGISTERED');
				return $this->redirecter('index.php?option=com_virtuemart&view=user&task=editaddresscart&addrtype=BT' , $redirectMsg);
			}
		}

		// Test Coupon
		if (!empty($this->couponCode)) {
			if (!class_exists('CouponHelper')) {
				require(VMPATH_SITE . DS . 'helpers' . DS . 'coupon.php');
			}

			if(!in_array($this->couponCode,$this->_triesValidateCoupon)){
				$this->_triesValidateCoupon[] = $this->couponCode;
			}
			if(count($this->_triesValidateCoupon)<8){
				$redirectMsg = CouponHelper::ValidateCouponCode($this->couponCode, $this->cartPrices['salesPrice']);
			} else{
				$redirectMsg = vmText::_('COM_VIRTUEMART_CART_COUPON_TOO_MANY_TRIES');
			}

			if (!empty($redirectMsg)) {
				$this->couponCode = '';
				$this->_inCheckOut = false;
				$this->setCartIntoSession();
				return $this->redirecter('index.php?option=com_virtuemart&view=cart'.$layoutName , $redirectMsg);
			}
		}
		$redirectMsg = '';

		//Test Shipment and show shipment plugin
		if (empty($this->virtuemart_shipmentmethod_id)) {
			return $this->redirecter('index.php?option=com_virtuemart&view=cart&task=edit_shipment' , $redirectMsg);
		} else {
			if (!class_exists('vmPSPlugin')) require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
			JPluginHelper::importPlugin('vmshipment');
			//Add a hook here for other shipment methods, checking the data of the choosed plugin
			$dispatcher = JDispatcher::getInstance();
			$retValues = $dispatcher->trigger('plgVmOnCheckoutCheckDataShipment', array(  $this));

			foreach ($retValues as $retVal) {
				if ($retVal === true) {
					break; // Plugin completed succesfull; nothing else to do
				} elseif ($retVal === false) {
					// Missing data, ask for it (again)
					return $this->redirecter('index.php?option=com_virtuemart&view=cart&task=edit_shipment' , $redirectMsg);
					// 	NOTE: inactive plugins will always return null, so that value cannot be used for anything else!
				}
			}
		}

		//Test Payment and show payment plugin
		if($this->cartPrices['salesPrice']>0.0){
			if (empty($this->virtuemart_paymentmethod_id)) {
				return $this->redirecter('index.php?option=com_virtuemart&view=cart&task=editpayment' , $redirectMsg);
			} else {
				if(!class_exists('vmPSPlugin')) require(JPATH_VM_PLUGINS.DS.'vmpsplugin.php');
				JPluginHelper::importPlugin('vmpayment');
				//Add a hook here for other payment methods, checking the data of the choosed plugin
				$dispatcher = JDispatcher::getInstance();
				$retValues = $dispatcher->trigger('plgVmOnCheckoutCheckDataPayment', array( $this));

				foreach ($retValues as $retVal) {
					if ($retVal === true) {
						break; // Plugin completed succesful; nothing else to do
					} elseif ($retVal === false) {
						// Missing data, ask for it (again)
						return $this->redirecter('index.php?option=com_virtuemart&view=cart&task=editpayment' , $redirectMsg);
						// 	NOTE: inactive plugins will always return null, so that value cannot be used for anything else!
					}
				}
			}
		}

		$validUserDataCart = self::validateUserData('cartfields',$this->cartfields,$this->_redirect);

		if($validUserDataCart!==true){
			if($this->_redirect){
				$this->_inCheckOut = false;
				$redirectMsg = null;
				return $this->redirecter('index.php?option=com_virtuemart&view=cart'.$layoutName , $redirectMsg);
			}
			$this->_blockConfirm = true;
		} else {
			//Atm a bit dirty. We store this information in the BT order_userinfo, so we merge it here, it gives also
			//the advantage, that plugins can easily deal with it.
			//$this->BT = array_merge((array)$this->BT,(array)$this->cartfields);
		}

		//Either we use here $this->_redirect, or we redirect always directly, atm we check the boolean _redirect
		if (count($this->cartProductsData) == 0) {
			$this->_inCheckOut = false;
			return $this->redirecter('index.php?option=com_virtuemart&view=cart', vmText::_('COM_VIRTUEMART_CART_NO_PRODUCT'));
		}

		//Show cart and checkout data overview
		if($this->_redirected){
			$this->_redirected = false;
		} else {
			$this->_inCheckOut = false;
		}

		if($this->_blockConfirm){
			$this->_dataValidated = false;
			$this->_inCheckOut = false;
			$this->setCartIntoSession(true);
			return $this->redirecter('index.php?option=com_virtuemart&view=cart'.$layoutName,'');
		} else {
			$this->_dataValidated = $this->getCartHash();

			$this->setCartIntoSession(true);
			if ($this->_redirect) {
				$app = JFactory::getApplication();
				$app->redirect(JRoute::_('index.php?option=com_virtuemart&view=cart'.$layoutName, FALSE), vmText::_('COM_VIRTUEMART_CART_CHECKOUT_DONE_CONFIRM_ORDER'));
			} else {
				return true;
			}
		}
	}

	/**
	 * Check if a minimum purchase value for this order has been set, and if so, if the current
	 * value is equal or hight than that value.
	 * @author Oscar van Eijk
	 * @return An error message when a minimum value was set that was not eached, null otherwise
	 */
	private function checkPurchaseValue() {

		$this->prepareVendor();
		if ($this->vendor->vendor_min_pov > 0) {
			$this->getCartPrices();
			if ($this->cartPrices['salesPrice'] < $this->vendor->vendor_min_pov) {
				if (!class_exists('CurrencyDisplay'))
				require(VMPATH_ADMIN . DS . 'helpers' . DS . 'currencydisplay.php');
				$currency = CurrencyDisplay::getInstance();
				return vmText::sprintf('COM_VIRTUEMART_CART_MIN_PURCHASE', $currency->priceDisplay($this->vendor->vendor_min_pov));
			}
		}
		return null;
	}

	/**
	 * Test userdata if valid
	 *
	 * @author Max Milbers
	 * @param String if BT or ST
	 * @param Object If given, an object with data address data that must be formatted to an array
	 * @return redirectMsg, if there is a redirectMsg, the redirect should be executed after
	 */
	public function validateUserData($type='BT', $obj = null,$redirect = false) {

		$usersModel = VmModel::getModel('user');
		if($obj==null){
			return $usersModel->validateUserData($this->{$type},$type,$redirect);
		} else {
			return $usersModel->validateUserData($obj,$type,$redirect);
		}
	}

	/**
	 * This function is called, when the order is confirmed by the shopper.
	 *
	 * Here are the last checks done by payment plugins.
	 * The mails are created and send to vendor and shopper
	 * will show the orderdone page (thank you page)
	 *
	 */
	function confirmedOrder() {
//vmdebug('confirmedOrder:',$this);
		//Just to prevent direct call
		if ($this->_dataValidated and $this->_confirmDone and !$this->_inCheckOut) {
//vmdebug('_inConfirm:',$this->_inConfirm);	
			if($this->_inConfirm) return false;

			//We set this in the trigger of the plugin. so old plugins keep the old behaviour
			$orderModel = VmModel::getModel('orders');
//vmdebug('$this:',$this);
			if(!$this->virtuemart_order_id){
				$this->virtuemart_order_id = $orderModel->createOrderFromCart($this);
				if (!$this->virtuemart_order_id) {
					$mainframe = JFactory::getApplication();
					//vmError('No order created '.$orderModel->getError());
					$mainframe->redirect(JRoute::_('index.php?option=com_virtuemart&view=cart', FALSE) );
				}
			}

			$orderDetails = $orderModel->getMyOrderDetails($this->virtuemart_order_id,$this->order_number,$this->order_pass);

			if(!$orderDetails or empty($orderDetails['details'])){
				echo vmText::_('COM_VIRTUEMART_CART_ORDER_NOTFOUND');
				return;
			}
			//$orderDetails['products'] = $this->products;

			$dispatcher = JDispatcher::getInstance();

			JPluginHelper::importPlugin('vmcalculation');
			JPluginHelper::importPlugin('vmcustom');
			JPluginHelper::importPlugin('vmshipment');
			JPluginHelper::importPlugin('vmpayment');

			$this->orderDetails = $orderDetails;

			$returnValues = $dispatcher->trigger('plgVmConfirmedOrder', array($this, $orderDetails));

//vmdebug('_confirmDone:',$this->_confirmDone);			
			if( $this->_confirmDone ) {
				$lifetime = (24 * 60 * 60) * 180; //180 days
				if(!class_exists('vmCrypt')){
					require(VMPATH_ADMIN.DS.'helpers'.DS.'vmcrypt.php');
				}
//vmdebug('customer_notified:',$this->customer_notified);
				if(!$this->customer_notified ) {
					$orderModel->notifyCustomer($this->virtuemart_order_id, $orderDetails);
				}
			}
			
			//NANDA
			//$orderModel->notifyCustomer($this->virtuemart_order_id, $orderDetails);

			// may be redirect is done by the payment plugin (eg: paypal)
			// if payment plugin echos a form, false = nothing happen, true= echo form ,
			// 1 = cart should be emptied, 0 cart should not be emptied
			$this->setCartIntoSession(false,true);

			return $this->virtuemart_order_id;
		}
	}

	public function getCartHash(){

		return md5(serialize($this->cartProductsData) .
		serialize($this->BT).
		($this->STsameAsBT) ? '': serialize($this->ST) .
		$this->STsameAsBT .
		$this->order_language.
		$this->selected_shipto.
		serialize($this->cartPrices).
		$this->virtuemart_shipmentmethod_id .
		(!empty($this->cartData['shipmentName']) ? $this->cartData['shipmentName'] : '') .
		$this->virtuemart_paymentmethod_id .
		(!empty($this->cartData['paymentName']) ? $this->cartData['paymentName'] : '') .
		$this->customer_number );
	}
	/**
	 * emptyCart: Used for payment handling.
	 *
	 * @author Valerie Cartan Isaksen
	 *
	 */
	public function emptyCart(){
		self::emptyCartValues($this);
	}

	/**
	 * emptyCart: Used for payment handling.
	 *
	 * @author Valerie Cartan Isaksen
	 *
	 */
	static public function emptyCartValues(&$cart, $session = true){

		//We delete the old stuff
		$cart->products = array();
		$cart->cartProductsData = array();
		$cart->cartData = array();
		$cart->cartPrices = array();
		$cart->cartfields = array();
		$cart->_inCheckOut = false;
		$cart->_dataValidated = false;
		$cart->_confirmDone = false;
		$cart->couponCode = '';
		$cart->order_language = '';
		$cart->virtuemart_shipmentmethod_id = 0; //OSP 2012-03-14
		$cart->virtuemart_paymentmethod_id = 0;
		$cart->order_number=null;
		$cart->_fromCart = false;
		$cart->_inConfirm = false;
		$cart->totalProduct=false;
		$cart->productsQuantity=array();
		$cart->virtuemart_order_id = null;

		if($session){
			$cart->deleteCart();
			$cart->setCartIntoSession(false,true);
		}
	}

	function saveCartFieldsInCart(){

		if (!class_exists('VirtueMartModelUserfields'))
			require(VMPATH_ADMIN . DS . 'models' . DS . 'userfields.php');
		$userFieldsModel = VmModel::getModel('userfields');

		$cartFields = $userFieldsModel->getUserFields(
			'cart'
			, array('delimiters' => true, 'captcha' => true, 'system' => false)
			, array('delimiter_userinfo', 'name','username', 'password', 'password2', 'address_type_name', 'address_type', 'user_is_vendor', 'agreed'));

		foreach ($cartFields as $fld) {
			if(!empty($fld->name)){
				$name = $fld->name;

				if($fld->type=='checkbox'){
					$tmp = vRequest::getInt($name,0);
				} else {
					$tmp = vRequest::getString($name,null);
				}

				if(isset($tmp)){
					if(!empty($tmp)){
						if(is_array($tmp)){
							$tmp = implode("|*|",$tmp);
						}
						if (version_compare(phpversion(), '5.4.0', '<')) {
							// php version isn't high enough
							$tmp = htmlspecialchars ($tmp,ENT_QUOTES,'UTF-8',false);	//ENT_SUBSTITUTE only for php5.4 and higher
						} else {
							$tmp = htmlspecialchars ($tmp,ENT_QUOTES|ENT_SUBSTITUTE,'UTF-8',false);
						}

						$tmp = (string)preg_replace('#on[a-z](.+?)\)#si','',$tmp);//replace start of script onclick() onload()...
					}

					$this->cartfields[$name] = $tmp;
					//vmdebug('Store $this->cartfields[$name] '.$name.' '.$tmp);
				}

			}
		}
		$this->BT = array_merge((array)$this->BT,(array)$this->cartfields);
		$this->setCartIntoSession();
	}

	function saveAddressInCart($data, $type, $putIntoSession = true,$prefix='') {

		// VirtueMartModelUserfields::getUserFields() won't work
		if(!class_exists('VirtueMartModelUserfields')) require(VMPATH_ADMIN.DS.'models'.DS.'userfields.php' );
		$userFieldsModel = VmModel::getModel('userfields');

		if ($type == 'STaddress' or $type == 'BTaddress'){
			vmTrace('STaddress found, seek and destroy');
		}
		$prepareUserFields = $userFieldsModel->getUserFieldsFor('cart',$type);

		if(!is_array($data)){
			$data = get_object_vars($data);
		}

		if ($type =='ST') {
			$this->STsameAsBT = 0;
		} else { // BT
			if(empty($data['email'])){
				$jUser = JFactory::getUser();
				$address['email'] = $jUser->email;
			}
		}

		$address = array();
		if(!class_exists('vmFilter'))require(VMPATH_ADMIN.DS.'helpers'.DS.'vmfilter.php');
		foreach ($prepareUserFields as $fld) {
			if(!empty($fld->name)){
				$name = $fld->name;

				if(!isset($data[$prefix.$name])){
					$tmp = vRequest::getString($prefix.$name,false);
					if($tmp){
						$data[$prefix.$name] = $tmp;
					}
					else if($fld->required and isset($this->{$type}[$name])){	//Why we have this fallback to the already stored value?
						$data[$prefix.$name] = $this->{$type}[$name];
					}
					/*if($fld->type=='text'){
					} else {
						vmdebug('my fld ',$fld);
					}*/
				}

				if(isset($data[$prefix.$name])){
					if(!empty($data[$prefix.$name])){
						if(is_array($data[$prefix.$name])){
							foreach($data[$prefix.$name] as $k=>$v){
								$data[$prefix.$name][$k] = self::filterCartInput($v);
							}
						} else {
							$data[$prefix.$name] = self::filterCartInput($data[$prefix.$name]);
						}
					}
					$address[$name] = $data[$prefix.$name];
				} else {
					//vmdebug('Data not found for type '.$type.' and name '.$prefix.$name.' ');
				}
			}
		}

		//dont store passwords in the session
		unset($address['password']);
		unset($address['password2']);

		$this->{$type} = $address;

		if($putIntoSession){
			$this->setCartIntoSession(true);
		}

	}

	private function filterCartInput($v){
		$v = vmFilter::hl( $v,array('deny_attribute'=>'*'));
		//to strong
		/* $value = preg_replace('@<[\/\!]*?[^<>]*?>@si','',$value);//remove all html tags  */
		//lets use instead
		$v = JComponentHelper::filterText($v);
		$v = (string)preg_replace('#on[a-z](.+?)\)#si','',$v);//replace start of script onclick() onload()...
		$v =  str_replace(array('"',"\t","\n","\r","\0","\x0B"),' ',trim($v));
		return (string)preg_replace('#^\'#si','',$v);
	}

	/**
	 * Returns ST address considering the set options, with fallback
	 * @author Max Milbers
	 */
	public function getST($name=0,$FBBT=true){

		$addr = $this->ST;

		if($this->STsameAsBT == 0){
			if($FBBT){
				if($name!==0){
					if(!isset($this->ST[$name])){
						$addr = $this->BT;
					}
				} else if($this->ST == 0){
					//vmdebug('getST ST=0, use BT');
					$addr = $this->BT;
				}
			}
		} else {
			//vmdebug('getST STsameAsBT is set, use BT');
			$addr = $this->BT;
		}

		if($name!==0){
			return isset($addr[$name]) ? $addr[$name] : '';
		} else {
			return $addr!==0 ? $addr : array();
		}

	}

	/**
	 * @author Valérie Isaksen, Max Milbers
	 * @param $type
	 * @return bool
	 */
	function checkAutomaticSelectedPlug($type){

		$vm_method_name = 'virtuemart_'.$type.'method_id';
		if (count($this->products) == 0  or  VmConfig::get('set_automatic_'.$type,'0')=='-1') {
			return false;
		}

		if (!class_exists('vmPSPlugin')) {
			require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
		}

		$counter=0;
		$dispatcher = JDispatcher::getInstance();
		$returnValues = $dispatcher->trigger('plgVmOnCheckAutomaticSelected'.ucfirst($type), array(  $this,$this->cartPrices, &$counter));

		$nb = 0;
		$method_id = 0;
		foreach ($returnValues as $returnValue) {
			if ( isset($returnValue )) {
				$nb ++;
				if ($returnValue) $method_id = $returnValue;
			}
		}

		$vm_autoSelected_name = 'automaticSelected'.ucfirst($type);
		if ($nb==1 && $method_id) {
			$this->$vm_method_name = $method_id;
			$this->$vm_autoSelected_name=true;
			$this->setCartIntoSession();
			//vmdebug('FOUND automatic SELECTED '.$type.' !!',$this->$vm_method_name);
			return true;
		} else {
			$this->$vm_autoSelected_name=false;
			return false;
		}

	}

	/*
	 * CheckAutomaticSelectedShipment
	* If only one shipment is available for this amount, then automatically select it
	* @deprecated
	* @author Valérie Isaksen
	*/
	function CheckAutomaticSelectedShipment() {
		return $this->checkAutomaticSelectedPlug('shipment');
	}

	/*
	 * CheckAutomaticSelectedPayment
	* If only one payment is available for this amount, then automatically select it
	* @deprecated
	* @author Valérie Isaksen
	*/
	function CheckAutomaticSelectedPayment() {
		return $this->checkAutomaticSelectedPlug('payment');
	}

	/**
	 * Function Description
	 *
	 * @author Max Milbers
	 * @access public
	 * @param array $cart the cart to get the products for
	 * @return array of product objects
	 */

	public function getCartPrices($force=false) {

		if(empty($this->cartPrices) or !$this->_calculated or $force){
			if(!class_exists('calculationHelper')) require(VMPATH_ADMIN.DS.'helpers'.DS.'calculationh.php');
			$calculator = calculationHelper::getInstance();

			$this->pricesCurrency = $calculator->_currencyDisplay->getCurrencyForDisplay();

			$calculator->getCheckoutPrices($this);

			//Fallback for old extensions
			$this->pricesUnformatted = $this->cartPrices;

			//We must do this here, otherwise if we have a product more than one time in the cart
			//it has always the same price
			foreach($this->products as $k => $product){
				$this->products[$k]->prices = &$product->allPrices[$product->selectedPrice];
			}
			$this->_calculated = true;
			//$this->setCartIntoSession();
		}
		return $this->cartPrices;
	}

	function prepareVendor(){
		if(empty($this->vendor)){
			$vendorModel = VmModel::getModel('vendor');
			$this->vendor = $vendorModel->getVendor($this->vendorId);
			$vendorModel->addImages($this->vendor,1);
			if (VmConfig::get('enable_content_plugin', 0)) {
				shopFunctionsF::triggerContentPlugin($this->vendor, 'vendor','vendor_terms_of_service');
			}
		}
	}

	function prepareCartData($force=true){

		$this->totalProduct = 0;
		if(count($this->products) != count($this->cartProductsData) or $this->_productAdded){
			$productsModel = VmModel::getModel('product');
			$this->totalProduct = 0;
			$this->productsQuantity = array();
			$customFieldsModel = VmModel::getModel('customfields');
			foreach($this->cartProductsData as $k =>&$productdata){
				$productdata = (array)$productdata;

				if(isset($productdata['virtuemart_product_id'])){
					if(empty($productdata['virtuemart_product_id']) or empty($productdata['quantity'])){
						unset($this->cartProductsData[$k]);
						continue;
					}
					$productdata['quantity'] = (int)$productdata['quantity'];
					//Important, must not use calculation, would lead to wrong prices, because the full cart is not know yet.
					$productTemp = $productsModel->getProduct($productdata['virtuemart_product_id'],TRUE,FALSE,TRUE,$productdata['quantity']);

					if(empty($productTemp->virtuemart_product_id)){
						vmError('prepareCartData virtuemart_product_id is empty','The product is no longer available');
						unset($this->cartProductsData[$k]);
						continue;
					}

					//Very important! must be cloned, else all products with same id get the same productCustomData due the product cache
					$productTemp->modificatorSum = null;
					$product = clone($productTemp);

					$productdata['virtuemart_product_id'] = (int)$productdata['virtuemart_product_id'];

					$product -> customProductData = $productdata['customProductData'];
					$product -> quantity = $productdata['quantity'];

					// No full link because Mail want absolute path and in shop is better relative path
					$product->url = JRoute::_('index.php?option=com_virtuemart&view=productdetails&virtuemart_product_id='.$product->virtuemart_product_id.'&virtuemart_category_id='.$product->virtuemart_category_id);//JHtml::link($url, $product->product_name);
					$product->cart_item_id = $k ;

					if ( VmConfig::get('oncheckout_show_images')){
						$productsModel->addImages($product,1);
					}

					$product->customfields = $customFieldsModel->getCustomEmbeddedProductCustomFields($product->allIds,0,1);

					$enough = $this->checkForQuantities($product,$product -> quantity);

					if($product->quantity<=0){
						unset($this->cartProductsData[$k]);
						continue;
					}
					$this->products[$k] = $product;
					$this->totalProduct += $product -> quantity;
					$productdata['quantity'] = $product -> quantity;

					if(isset($this->productsQuantity[$product->virtuemart_product_id])){
						$this->productsQuantity[$product->virtuemart_product_id] += $product -> quantity;
					} else {
						$this->productsQuantity[$product->virtuemart_product_id] = $product -> quantity;
					}

					$product = null;
				} else {
					unset($this->cartProductsData[$k]);
					vmError('prepareCartData $productdata[virtuemart_product_id] was empty');
				}
			}
			$this->setCartIntoSession();
		} else {

		}

		$this->getCartPrices($force);

		if(!class_exists('vmPSPlugin')) require(JPATH_VM_PLUGINS.DS.'vmpsplugin.php');
		JPluginHelper::importPlugin('vmpayment');
		$dispatcher = JDispatcher::getInstance();
		$returnValues = $dispatcher->trigger('plgVmgetPaymentCurrency', array( $this->virtuemart_paymentmethod_id, &$this->paymentCurrency));

		$this->_productAdded = false;
		return $this->cartData ;

	}

	/*
	 * Trigger to place Coupon, payment, shipment advertisement on the cart
	 */
	public function getCheckoutAdvertise() {

		if(!$this->cartAdv){
			$this->cartAdv=array();
			JPluginHelper::importPlugin('vmextended');
			JPluginHelper::importPlugin('vmcoupon');
			JPluginHelper::importPlugin('vmshipment');
			JPluginHelper::importPlugin('vmpayment');
			JPluginHelper::importPlugin('vmuserfield');
			$dispatcher = JDispatcher::getInstance();
			$returnValues = $dispatcher->trigger('plgVmOnCheckoutAdvertise', array( $this, &$this->cartAdv));
		}
		return $this->cartAdv;
	}

	/** Checks if the quantity is correct
	 *
	 * @author Max Milbers
	 */
	private function checkForQuantities($product, &$quantity=0) {

		$dispatcher = JDispatcher::getInstance();
		JPluginHelper::importPlugin('vmcustom');
		// return null to proceed with further VM rules
		// return true to not validate the quantity with OPC or VM
		// return false to return errorMsg
		//if to allow to adjust the current quantity
		$adjustQ = false; 
		$errorMsg = ''; 
		$retValues = $dispatcher->trigger('plgVmOnCheckoutCheckStock', array(  &$this, &$product, &$quantity, &$errorMsg, &$adjustQ));
		
		foreach ($retValues as $v) {
			if ($v === false) {
				vmInfo($errorMsg,$product->product_name);
				return false; 
			}
			if ($v === true) {
				return true; 
			}
		}
	
		$stockhandle = VmConfig::get('stockhandle_products', false) && $product->product_stockhandle ? $product->product_stockhandle : VmConfig::get('stockhandle','none');
		$mainframe = JFactory::getApplication();
		// Check for a valid quantity
		if (!is_numeric( $quantity)) {
			$errorMsg = vmText::_('COM_VIRTUEMART_CART_ERROR_NO_VALID_QUANTITY', false);
			vmInfo($errorMsg,$product->product_name);
			return false;
		}
		// Check for negative quantity
		if ($quantity < 1) {
			$errorMsg = vmText::_('COM_VIRTUEMART_CART_ERROR_NO_VALID_QUANTITY', false);
			vmInfo($errorMsg,$product->product_name);
			return false;
		}

		// Check to see if checking stock quantity
		if ($stockhandle!='none' && $stockhandle!='risetime') {

			$productsleft = $product->product_in_stock - $product->product_ordered;

			// TODO $productsleft = $product->product_in_stock - $product->product_ordered - $quantityincart ;
			if ($quantity > $productsleft ){
				vmdebug('my products left '.$productsleft.' and my quantity '.$quantity);
				if($productsleft>0){
					$quantity = $productsleft;
					$product->errorMsg = vmText::sprintf('COM_VIRTUEMART_CART_PRODUCT_OUT_OF_QUANTITY',$product->product_name,$quantity);
					vmError($product->errorMsg);
				} else {
					$quantity = 0;
					$product->errorMsg = vmText::_('COM_VIRTUEMART_CART_PRODUCT_OUT_OF_STOCK');
					vmError($product->errorMsg); // Private error retrieved with getError is used only by addJS, so only the latest is fine
					// todo better key string
					vmInfo($product->errorMsg. ' '.$product->product_name);
					return false;
				}
			}
		}

		// Check for the minimum and maximum quantities
		$min = $product->min_order_level;
		if ($min != 0 && $quantity < $min){
			$quantity = $min;
			$errorMsg = vmText::sprintf('COM_VIRTUEMART_CART_MIN_ORDER', $min, $product->product_name);
			vmInfo($errorMsg,$product->product_name);
			return false;
		}

		$max = $product->max_order_level;
		if ($max != 0 && $quantity > $max) {
			$quantity = $max;
			$errorMsg = vmText::sprintf('COM_VIRTUEMART_CART_MAX_ORDER', $max, $product->product_name);
			vmInfo($errorMsg,$product->product_name);
			return false;
		}

		$step = $product->step_order_level;
		if ($step != 0 && ($quantity%$step)!= 0) {
			$quantity = $quantity + ($quantity%$step);
			$errorMsg = vmText::sprintf('COM_VIRTUEMART_CART_STEP_ORDER', $step);
			vmInfo($errorMsg,$product->product_name);
			return false;
		}
		return true;
	}

	// Render the code for Ajax Cart
	function prepareAjaxData($withProductImages=false){

		$this->prepareCartData(false);
		$data = new stdClass();
		$data->products = array();
		$data->totalProduct = 0;

		//OSP when prices removed needed to format billTotal for AJAX
		if (!class_exists('CurrencyDisplay'))
			require(VMPATH_ADMIN . DS . 'helpers' . DS . 'currencydisplay.php');
		$currencyDisplay = CurrencyDisplay::getInstance();

		foreach ($this->products as $i=>$product){

			//Create product URL
			$url = JRoute::_('index.php?option=com_virtuemart&view=productdetails&virtuemart_product_id='.$product->virtuemart_product_id.'&virtuemart_category_id='.$product->virtuemart_category_id, FALSE);
			$data->products[$i]['product_name'] = JHtml::link($url, $product->product_name);

			if(!class_exists('VirtueMartModelCustomfields'))require(VMPATH_ADMIN.DS.'models'.DS.'customfields.php');

			//  custom product fields display for cart
			$data->products[$i]['customProductData'] = VirtueMartModelCustomfields::CustomsFieldCartModDisplay($product);
			$data->products[$i]['product_sku'] = $product->product_sku;
			$data->products[$i]['prices'] = $currencyDisplay->priceDisplay( $product->allPrices[$product->selectedPrice]['subtotal']);
			if($withProductImages and !empty($product->images[0])) $data->products[$i]['image']= $product->images[0]->displayMediaThumb ('', FALSE);

			// other possible option to use for display
			$data->products[$i]['subtotal'] = $currencyDisplay->priceDisplay($product->allPrices[$product->selectedPrice]['subtotal']);
			$data->products[$i]['subtotal_tax_amount'] = $currencyDisplay->priceDisplay($product->allPrices[$product->selectedPrice]['subtotal_tax_amount']);
			$data->products[$i]['subtotal_discount'] = $currencyDisplay->priceDisplay( $product->allPrices[$product->selectedPrice]['subtotal_discount']);
			$data->products[$i]['subtotal_with_tax'] = $currencyDisplay->priceDisplay($product->allPrices[$product->selectedPrice]['subtotal_with_tax']);

			// UPDATE CART / DELETE FROM CART
			$data->products[$i]['quantity'] = $product->quantity;
			$data->totalProduct += $product->quantity ;

		}

		if(empty($this->cartPrices['billTotal']) or $this->cartPrices['billTotal'] < 0){
			$this->cartPrices['billTotal'] = 0.0;
		}

		$data->billTotal = $currencyDisplay->priceDisplay( $this->cartPrices['billTotal'] );
		$data->billTotal_tax_amount = $currencyDisplay->priceDisplay( $this->cartPrices['taxAmount'] );
		$data->billTotal_net = $currencyDisplay->priceDisplay( $this->cartPrices['priceWithoutTax'] );
		$data->billTotal_discounted_net = $currencyDisplay->priceDisplay( $this->cartPrices['discountedPriceWithoutTax'] );
		//end
		$data->dataValidated = $this->_dataValidated ;

		if ($data->totalProduct>1) $data->totalProductTxt = vmText::sprintf('COM_VIRTUEMART_CART_X_PRODUCTS', $data->totalProduct);
		else if ($data->totalProduct == 1) $data->totalProductTxt = vmText::_('COM_VIRTUEMART_CART_ONE_PRODUCT');
		else $data->totalProductTxt = vmText::_('COM_VIRTUEMART_EMPTY_CART');
		if (false && $data->dataValidated == true) {
			$taskRoute = '&task=confirm';
			$linkName = vmText::_('COM_VIRTUEMART_ORDER_CONFIRM_MNU');
		} else {
			$taskRoute = '';
			$linkName = vmText::_('COM_VIRTUEMART_CART_SHOW');
		}

		$data->cart_show = '<a style ="float:right;" href="'.JRoute::_("index.php?option=com_virtuemart&view=cart".$taskRoute,$this->useSSL).'" rel="nofollow" >'.$linkName.'</a>';
		$data->billTotal = vmText::sprintf('COM_VIRTUEMART_CART_TOTALP',$data->billTotal);

		return $data ;
	}
}
